FUNCTION CONS_COO,coo,xy,lon=lon,lat=lat

;+
; NAME:
;	CONS_COO
; PURPOSE:
;	Return a set of X or Y pixel values given an image with astrometry, 
;	and either
;	(1) a set of Y OR X pixel values, and a scalar right ascension, or
;	(2) a set of right ascension values, and a scalar Y OR X value.
;
;	In usage (1), CONS_COO can be used to determine the (X,Y) values
;	of a line of constant right ascension.  In usage (2), CONS_COO can
;	used to determine the X positions of specified RA values, along a
;	line of constant Y.
;
; CALLING SEQUENCE:
;	X = CONS_COO( COO, xy, h)
; INPUTS:
;	coo -  coordinate value in RADIANS (0 < RA < 2*PI).  If Y is a
;		vector, then RA must be a scalar
;	XY -   Specified Y pixel value(s) for line of constant right ascension
;		If RA is a vector, then Y must be a scalar
;	CD -  2 x 2 Matrix in RADIANS giving the values of the CD astrometry
;		parameters.
;	CRPIX-2 element vector giving the X and Y coordinates of the 
;		reference pixel (exactly as given in a FITS header).
;	CRVAL-2 element vector giving right ascension and declination in
;		RADIANS of the reference pixel.
;
; OUTPUTS
;	X   - Computed set of X pixel values.   The number of elements of X
;		is the maximum of the number of elements of RA and Y.
;
; RESTRICTIONS:
;	Program will have difficulty converging for declination values near
;	90.    A tangent projection is assumed
;
; REVISION HISTORY:
;	Written, Wayne Landsman  STX Co.	April, 1988
;	Algorithm adapted from AIPS memo No. 27 by Eric Griessen
;-

On_error,2

@proj_common.com

if ( N_params() LT 2 ) then begin
	print,'Syntax - X = CONS_COO( coo, XY)'
	print,'Key-words  = [,/lon][,/lat]'
        return, 0
endif

;h2pix,h,/silent
cd=rota
crval=[crval1,crval2]
crpix=[crpix1,crpix2]

IF proj EQ 'TAN' THEN BEGIN
  IF keyword_set(lon) THEN BEGIN
    yy=xy-(crpix(1)-1.)    ;New coordinate origin, Unit pixel offset in CRPIX
    cdi=invert( cd )          ;Griesen uses invert of CD matrix
    delra=coo-crval(0)
    cdelra=cos(delra )    &    sdelra = sin( delra )
    cdel0= cos(crval(1))  &    sdel0 = sin( crval(1) )
    delta=atan(sdel0*cdelra*cdi(1,1)-sin(delra)*cdi(1,0)+yy*cdelra*cdel0, $
            cdel0*cdi(1,1) - yy*sdel0)
    coo2pix2,coo*!radeg,delta*!radeg,res,y1,/silent
;    ad2xy,coo,delta,cd,crpix,crval,res,y1
;    res=res+1					;now IDL pixel
  ENDIF ELSE BEGIN
    xx = xy-(crpix(0)-1.)          ;New coordinate origin
    cdi = invert(cd)              ;Greisen uses invert of the CD matrix
    sdel0 = sin(crval(1)) & cdel0 = cos(crval(1))
    a = cdi(0,0)
    b = xx*cdel0 + sdel0*cdi(0,1)
    sign = 2*( a GT 0 ) - 1
    alpha = crval(0) + atan(b/a) + $
    sign*asin( tan(coo)* (xx*sdel0-cdel0*cdi(0,1) )/ (sqrt(a^2+b^2)) )
    coo2pix2,alpha*!radeg,coo*!radeg,x1,res,/silent
;    ad2xy, alpha, coo, cd, crpix, crval, x1, res
;    res=res+1					;Now IDL pixel
  ENDELSE
ENDIF

IF proj EQ 'GLS' THEN BEGIN
  IF keyword_set(lon) THEN BEGIN
    cdi = invert(cd)              ;Greisen uses invert of the CD matrix
    yy=xy-(crpix(1)-1.)
    delra=coo-crval(0)
    del=crval(1)+yy/cdi(1,1)
    FOR i=0,5 DO BEGIN
      old_cos=cos(del)
      delta=crval(1)-(yy+delra*old_cos*cdi(0,0)/cdi(1,1))
      del=delta
    ENDFOR
    coo2pix2,coo*!radeg,delta*!radeg,res,y1
  ENDIF ELSE BEGIN
    cdi = invert(cd)              ;Greisen uses invert of the CD matrix
    xx=xy-(crpix(0)-1.)
    cdi = invert(cd)
    yy=((coo-crval(1))+xx*cdi(1,0))/cdi(1,1)
    alpha=crval(0)+(xx*cdi(1,1)+yy*cdi(1,0))/cos(coo)
    coo2pix2,alpha*!radeg,coo*!radeg,x1,res
  ENDELSE
END

;IF proj EQ 'CAR' THEN BEGIN
;  IF keyword_set(lon) THEN BEGIN
    
;  ENDIF ELSE BEGIN
;  ENDELSE
;ENDIF

RETURN, res

END
