FUNCTION read_co_survey,hh,cobe=cobe

;+
; NAME:
;	read_co_survey
; PURPOSE:
;       Read Galactic CO survey data
; CATEGORY:
;       General
; CALLING SEQUENCE:
;	   data=read_co_survey(header,cobe=cobe)
; INPUTS:
;       header = fits header to which reprojection is to be done (including velocity info)
; OPTIONAL INPUT PARAMETERS:
;       None
; OUTPUTS:
;	   data        = CO data cube
; OPTIONAL OUTPUT PARAMETERS:
;	   None
; ACCEPTED KEY-WORDS:
;	  cobe           = if set, sixpack cobe header is assumed
; COMMON BLOCKS:
;	  CO_data
; SIDE EFFECTS:
;      requires the following IDL system variables:
;          !cosurvey_data_dir
;      assumes that the data cube file name is CO_survey.fits
; RESTRICTIONS:
; PROCEDURE:
;
; EXAMPLES
;    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard 20/10/00
;-

common CO_data,dco,hco

dir_co=!cosurvey_data_dir

file_co=dir_co+'CO_survey.fits'
IF defined(dco) EQ 0 THEN dco=readfits(file_co,hco)
Nx=sxpar(hco,'NAXIS1')
Ny=sxpar(hco,'NAXIS2')
Nv=sxpar(hco,'NAXIS3')
one_spectrum=fltarr(1,1,Nv)

NNx=sxpar(hh,'NAXIS1')
NNy=sxpar(hh,'NAXIS2')
NNv=sxpar(hh,'NAXIS3')
result=fltarr(NNx,NNy,NNv)+!indef

;== compute l,b coordinates for the cube
ctype1=sxpar(hh,'CTYPE1')
coo_sys=strmid(ctype1,0,4)
IF coo_sys NE 'GLON' THEN BEGIN
  CASE coo_sys OF
    'RA--': BEGIN
      equi=sxpar(hh,'EQUINOX')
      create_coo2,hh,ra,dec,/silent
      IF equi NE astron_default_equinox() THEN BEGIN
        precess_array,ra,dec,equi,astron_default_equinox()
      ENDIF
      euler,ra,dec,ll,bb,1
      ra=0 & dec=0
    END
    'ELON': BEGIN
      create_coo2,hh,elon,elat,/silent
      IF not keyword_set(cobe) THEN BEGIN
        euler,elon,elat,ll,bb,5
        elon=0 & elat=0
      ENDIF ELSE BEGIN
        ;== this is to fix the 1-pixel difference in coordinates when
        ;== using COBE sixpack
        euler,elon,elat,ra,dec,4
        elon=0 & elat=0
        IF astron_default_equinox() NE 2000. THEN $
                 precess_array,ra,dec,2000.,astron_default_equinox()
        euler,ra,dec,ll,bb,1
        ra=0 & dec=0
      ENDELSE
    END
  ENDCASE
ENDIF ELSE BEGIN
  create_coo2,hh,ll,bb,/silent
ENDELSE

h2pix,hco,/silent
coo2pix2,ll,bb,ipix,jpix,/silent
ipix=round(ipix) & jpix=round(jpix)

make_axis,hh,xx,yy,vels
dv_header=sxpar(hh,'CDELT3')
  
ind=where(ipix GE 0 AND ipix LE (Nx-1) AND jpix GE 0 AND jpix LE (Ny-1),count)
;stop
ij=index2ij(ind,[NNx,NNy])
old_perc=0
IF count NE 0 THEN BEGIN
  FOR i=0L,count-1 DO BEGIN
    one_spectrum(0,0,*)=dco(ipix(ind(i)),jpix(ind(i)),*)
    indok=where(one_spectrum NE !indef,countok)
    IF countok NE 0 THEN BEGIN
      FOR k=0L,NNv-1 DO BEGIN
        vr=[vels(k)-dv_header/2.,vels(k)+dv_header/2.]
        ii=integrated_intensity(one_spectrum,hco,vrange=vr,/silent)
;        stop
        result(ij(i,0),ij(i,1),k)=ii
      ENDFOR
      perc=fix(1.*(i+1)/(1.*(count-1))*100.)
      IF perc mod 2 EQ 0 AND old_perc NE perc THEN BEGIN
        print,'Achieved '+strtrim(perc,2)+'% @'+systime(0)
        im=integrated_intensity(result,hh,vr=minmax(vels),/silent)
        tvscl,im>0
        old_perc=perc
      ENDIF
    ENDIF
  ENDFOR
ENDIF

RETURN,result
END
