FUNCTION read_dwingeloo,lon1,lon2,lat1,lat2,vel1,vel2, $
         header=header,swap=swap,pick_dir=pick_dir,show=show,dist_arr=dist_arr, $
         h_in=h_in,catpos=catpos,integrated=integrated,cobe=cobe, $
         l_data=l_data,b_data=b_data

;+
; NAME:
;	read_dwingeloo
; PURPOSE:
;       Read HI dwingeloo data
; CATEGORY:
;       General
; CALLING SEQUENCE:
;	   data=read_dwingeloo([lon1,lon2,lat1,lat2,vel1,vel2,] , $
;                            header=header,swap=swap,dir=dir,show=show, $
;                            dist_arr=dist_arr,h_in=h_in)
; INPUTS:
;
; OPTIONAL INPUT PARAMETERS:
;       lon1,lon2   = longitude range to be extracted (degrees).
;                     Not used if h_in is set.
;       lat1,lat2   = latitude range to be extracted (degrees).
;                     Not used if h_in is set.
;       vel1,vel2   = velocity range to be extracted (km/s).
;                     Not used if h_in is set.
; OUTPUTS:
;	   data        = HI data cube
;	   header      = HI data fits header
; OPTIONAL OUTPUT PARAMETERS:
;	   dist_arr    = distance between each pixel and nearest HI spectrum used (degrees)
; ACCEPTED KEY-WORDS:
;	  swap           = if set, data are byte swapped
;       pick_dir      = if set, directory containing HI data is asked for
;       h_in          = if set, the data is reprojected on that header.
;                       Must be a cube header.
; COMMON BLOCKS:
;	  HI_parameters
; SIDE EFFECTS:
;      requires the following IDL system variables:
;          !dwingeloo_param_dir
;          !dwingeloo_data_dir
;      assumes that the data cube name is CUBE.CUB
; RESTRICTIONS:
; PROCEDURE:
;      If lon1, ... vel2 are given, they are used to extract the proper cube from the data
;      and a Galactic (cartesian) fits header is constructed and returned.
;      If a header is given, it is used to project de data. If CDELT3 is equal to the velocity
;      resolution of the Dwingeloo survey (1.0305165) within 1%, spectra are simply extracted in
;      the velocity range given in the header. Otherwise, integrated intensities are calculated
;      in each v bin defined by the header. 
;      The nearest HI spectrum is always used. No spatial interpolation in the original data is
;      ever done
; EXAMPLES
;    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard 13/07/00 adapted from a program by F. Boulanger
;-

;common HI_parameters,HIparam,hparam,catposind
common HI_parameters,HIparam,hparam,HIindex

dvHI = 1.0305165   ; espacement entre deux canaux consecutifs

;== Read catalog index and scaling parameters, if not already in the common
param_dir=!dwingeloo_param_dir
IF not defined(HIparam) THEN BEGIN
;  param_file='Xcaret6G:Data:leiden:parametres.fits'
  param_file=param_dir+'parametres.fits'
  HIparam=readfits(param_file,hparam)
  print,'HI parameters have been read from '+param_file
;  param_file='Xcaret6G:Data:leiden:HI_index.fits
  param_file=param_dir+'HI_index.fits'
  HIindex=readfits(param_file,hp)
  print,'HI index read from '+param_file
ENDIF

;== Open the file containing the HI cube and do the assoc
read_dir=!dwingeloo_data_dir
IF keyword_set(pick_dir) THEN BEGIN
  read_dir=dialog_pickfile(/directory)
ENDIF

read_file=read_dir+'CUBE.CUB'
f=findfile(read_file,count=c)
IF c EQ 0 THEN BEGIN
  message,'File: '+read_file+' was not found.'
  GOTO,sortie
ENDIF

IF NOT keyword_set(h_in) THEN BEGIN
  dv_header=dvHI
  ; conversion des parametres du sous-cube en indices
  ilon1 = fix((lon1-0.5)*2.)
  ilon2 = fix((lon2-0.5)*2.)
  dlon=0.5
  lonstart=max([lon1,lon2])
  IF lon1 LE lon2 THEN BEGIN
    ilon_from=ilon1
    ilon_to=ilon2
    Nlon=(lon2-lon1)/dlon
  ENDIF ELSE BEGIN
    ilon_from=ilon2
    ilon_to=ilon1  
    Nlon=(360.-lon1+lon2)/dlon
  ENDELSE
  IF lonstart GT 180. THEN lonstart=lonstart-360.
  ilat1 = fix((lat1+90.)*2.)
  ilat2 = fix((lat2+90.)*2.)
  Nlat=ilat2-ilat1+1

  ;=== Make a header for the final cube
  chv1 = fix(vel1/dvHI + 511.5)
  chv2 = fix(vel2/dvHI + 511.5)
  data_cube = fltarr(Nlon,Nlat,chv2-chv1+1)
  mkhdr,h,data_cube
  ctype1='GLON-CAR' & ctype2='GLAT-CAR' & ctype3='VELO-CAR'
  sxaddpar,h,'CTYPE1',ctype1 & sxaddpar,h,'CTYPE2',ctype2
  sxaddpar,h,'CTYPE3',ctype3
  crpix1=1 & crpix2=1 & crpix3=1
  sxaddpar,h,'CRPIX1',crpix1 & sxaddpar,h,'CRPIX2',crpix2
  sxaddpar,h,'CRPIX3',crpix3
  cdelt1=-0.5 & cdelt2=0.5 & cdelt3=dv_header
  sxaddpar,h,'CDELT1',cdelt1 & sxaddpar,h,'CDELT2',cdelt2
  sxaddpar,h,'CDELT3',cdelt3
  crota1=0. & crota2=0. & crota3=0.
  sxaddpar,h,'CROTA1',crota1 & sxaddpar,h,'CROTA2',crota2
  sxaddpar,h,'CROTA3',crota3
  crval1=lonstart & crval2=lat1 & crval3=vel1
  sxaddpar,h,'CRVAL1',crval1 & sxaddpar,h,'CRVAL2',crval2
  sxaddpar,h,'CRVAL3',crval3
  sxaddpar,h,'BUNITS','K'
  chv1 = fix(vel1/dvHI + 511.5)
  chv2 = fix(vel2/dvHI + 511.5)
  Nv=chv2-chv1+1
ENDIF ELSE BEGIN
  h=h_in
  Nlon=sxpar(h,'NAXIS1') & Nlat=sxpar(h,'NAXIS2') & Nv=sxpar(h,'NAXIS3')
  dv_header=sxpar(h,'CDELT3')
ENDELSE
make_axis,h,xx,yy,vels
vel1=min(vels) & vel2=max(vels)
;Modified 21/07/01 to have it work with cdelt3=1.0305165
;chv1 = fix(vel1/dvHI + 511.5)
;chv2 = fix(vel2/dvHI + 511.5)
chv1 = round(vel1/dvHI + 511.5)
chv2 = round(vel2/dvHI + 511.5)

; conversion vitesse en numero de canal; ch 511.5 correspond a vlsr = 0 km/s
; en comptant les canaux a partir de 0 jusqu'a 1023
data_cube = fltarr(Nlon,Nlat,Nv)+!indef
dist_arr=fltarr(Nlon,Nlat)
catpos=lonarr(Nlon,Nlat)+!indef

;== compute l,b coordinates for the cube
ctype1=sxpar(h,'CTYPE1')
coo_sys=strmid(ctype1,0,4)
IF coo_sys NE 'GLON' THEN BEGIN
  CASE coo_sys OF
    'RA--': BEGIN
      equi=sxpar(h,'EQUINOX')
      create_coo2,h,ra,dec,/silent
      IF equi NE astron_default_equinox() THEN BEGIN
        precess_array,ra,dec,equi,astron_default_equinox()
      ENDIF
      euler,ra,dec,ll,bb,1
    END
    'ELON': BEGIN
      create_coo2,h,elon,elat,/silent
      IF not keyword_set(cobe) THEN BEGIN
        euler,elon,elat,ll,bb,5
      ENDIF ELSE BEGIN
        ;== this is to fix the 1-pixel difference in coordinates when
        ;== using COBE sixpack
        euler,elon,elat,ra,dec,4
        IF astron_default_equinox() NE 2000. THEN $
                 precess_array,ra,dec,2000.,astron_default_equinox()
        euler,ra,dec,ll,bb,1
      ENDELSE
    END
  ENDCASE
ENDIF ELSE BEGIN
  create_coo2,h,ll,bb,/silent
ENDELSE

IF not keyword_set(silent) THEN BEGIN
  print,ll(0,Nlat/2)
  print,ll(Nlon-1,Nlat/2)
  help,data_cube
ENDIF

;== compute HI parameter arrays
h2pix,hparam,/silent
coo2pix2,ll,bb,ii,jj,/silent
iiPix=round(ii) & jjpix=round(jj)
parm0=my_bilinear2(HIparam(*,*,0),iipix,jjpix)
parm1=my_bilinear2(HIparam(*,*,1),iipix,jjpix)
image_cont20,parm1,h,/square,/silent
catpos=my_bilinear2(HIindex,iipix,jjpix)
h2pix,hparam,/silent
;These are the coordinates of actual spectra
pix2coo2,iipix,jjpix,l_data,b_data,/silent
gcirc,0,ll/!radeg,bb/!radeg,l_data/!radeg,b_data/!radeg,dist_arr
dist_arr=dist_arr*!radeg

;=== Open the data file
get_lun,unit
openr, unit, read_file
header  = bytarr(191) & spec = intarr(1024) & next = lonarr(1)
st={header:header,spec:spec,next:next}
array_desc=st
a=assoc(unit,array_desc,/packed)

;== do a fake header for 1 spectrum
v=a(0)
NNv=(size(v.spec))(1)
spec=fltarr(1,1,NNv)
spec(0,0,*)=v.spec
mkhdr,h_data,spec
cdelt3=dvHI
sxaddpar,h_data,'CDELT3',cdelt3
crpix3=1
sxaddpar,h_data,'CRPIX3',crpix3
crval3=-511.5*dvHI
sxaddpar,h_data,'CRVAL3',crval3
one_cube=fltarr(1,1,NNv)

make_axis,h_data,xx,yy,vvels
;stop
vrmin=vels-dv_header/2.
vrmax=vels+dv_header/2.
ptrv=ptrarr(Nv)
FOR k=0,Nv-1 DO BEGIN
  ind=where(vvels GE vrmin(k) AND vvels LT vrmax(k),count)
  print,vrmin(k),vrmax(k),count
  ptrv(k)=ptr_new(ind)
ENDFOR

ind=where(catpos NE !indef)
ccatpos=catpos(ind)
order=uniq(ccatpos,sort(ccatpos))
spec2read=ccatpos(order)
Nspec=(size(spec2read))(1)

old_perc=0
;=== put each spectrum into the output
IF abs(dv_header-dvHI) LE 1E-2 THEN BEGIN
  FOR i=0L,Nspec-1 DO BEGIN
    ispec=spec2read(i)
    v=a(ispec) & spec=v.spec
    IF keyword_set(swap) THEN byteorder,spec
    sspec=spec(chv1:chv2)
    ind=where(catpos EQ ispec)
    ij=index2ij(ind,[Nlon,Nlat])
    Nij=(size(ij))(1)
    FOR l=0,Nij-1 DO data_cube(ij(l,0),ij(l,1),*)=sspec
    IF keyword_set(show) THEN plot,sspec
    perc=fix(1.*(i+1)/(1.*Nspec)*100.)
    IF perc mod 2 EQ 0 AND old_perc NE perc THEN BEGIN
      print,'Achieved '+strtrim(perc,2)+'% @'+systime(0)
      old_perc=perc
    ENDIF
  ENDFOR
  FOR k=0,Nv-1 DO BEGIN
    im=data_cube(*,*,k)
    ind=where(im NE !indef,count)
    IF count NE 0 THEN BEGIN
      im(ind)=im(ind)*parm1(ind)+parm0(ind)
    ENDIF
    data_cube(*,*,k)=im
  ENDFOR
ENDIF ELSE BEGIN              ;This is to compute integrated intensities
  FOR i=0L,Nspec-1 DO BEGIN
;    stop
    ispec=spec2read(i)
    v=a(ispec) & spec=v.spec   ;This is the spectrum considered
    IF keyword_set(swap) THEN byteorder,spec
    ind=where(catpos EQ ispec)
    ij=index2ij(ind,[Nlon,Nlat])  ;This is where the spectrum belongs in array
    Nij=(size(ij))(1)
    l=0
;    sspec=spec*parm1(ij(l,0),ij(l,1))+parm0(ij(l,0),ij(l,1))
;    plot,vvels,sspec,xrange=[min(vels-dv_header/2.), $
;         max(vels+dv_header/2.)],/xsty    
    FOR k=0,Nv-1 DO BEGIN
      IF not keyword_set(integrated) THEN BEGIN
        FOR l=0,Nij-1 DO BEGIN
          data_cube(ij(l,0),ij(l,1),k)=total(spec(*ptrv(k)))*dvhi
        ENDFOR
      ENDIF ELSE BEGIN
        one_cube(0,0,*)=spec
        vr=[vels(k)-dv_header/2.,vels(k)+dv_header/2.]
        ii=integrated_intensity(one_cube,h_data,vrange=vr,/silent)
        FOR l=0,Nij-1 DO data_cube(ij(l,0),ij(l,1),k)=ii
      ENDELSE
    ENDFOR    
    perc=fix(1.*(i+1)/(1.*Nspec)*100.)
    IF perc mod 2 EQ 0 AND old_perc NE perc THEN BEGIN
      print,'Achieved '+strtrim(perc,2)+'% @'+systime(0)
      tvscl,data_cube(*,*,Nv/2)
      old_perc=perc
    ENDIF ELSE BEGIN
    ENDELSE
  ENDFOR
  FOR k=0,Nv-1 DO BEGIN
    im=data_cube(*,*,k)
    ind=where(im NE !indef,count)
    IF count NE 0 THEN BEGIN
      im(ind)=im(ind)*parm1(ind)+parm0(ind)*dv_header
    ENDIF
    data_cube(*,*,k)=im
  ENDFOR
;
;  FOR k=0,Nv-1 DO data_cube(*,*,k)=data_cube(*,*,k)*parm1+parm0*dv_header
  ind=where(parm1 EQ !indef,count)
  IF count NE 0 THEN BEGIN
    FOR k=0,Nv-1 DO BEGIN
      im=data_cube(*,*,k)
      im(ind)=!indef
      data_cube(*,*,k)=im      
    ENDFOR
  ENDIF
  sxaddpar,h,'BUNITS','K km/s'
ENDELSE

close,unit
free_lun,unit
header=h

RETURN,data_cube

sortie:

END
