;+
; NAME:
;       cw_pickvalues
; PURPOSE:
;       pick values from a list of strings
; CATEGORY:
;	JPBLib Library Compound widget.
; CALLING SEQUENCE:
;	w=cw_pickvalues(parent,uvalue=uvalue,value=value,already_selected=already_selected)
; INPUTS:
;     parent           : The ID of the parent widget.
; KEYWORD PARAMETERS:
;     uvalue           : User value for the widget
;     value            : list of selectable strings
;     already_selected : list of already selected strings
; OUTPUTS:
;      w               : The ID of the created widget is returned.
; COMMON BLOCKS:
;	None.
; SIDE EFFECTS:
;    None
; PROCEDURE:
;	WIDGET_CONTROL, id, SET_VALUE=value  sets value
;	WIDGET_CONTROL, id, GET_VALUE=var    retrieves the selected strings.
; MODIFICATION HISTORY:
;    Created by J.P. Bernard on 16/01/00
;-

PRO pickvalues_set_value, id, selection_values

ON_ERROR, 2

; Retrieve the state.
stash = WIDGET_INFO(id, /CHILD)
WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY

widget_control,state.widgets.input_list,set_value=selection_values
widget_control,state.widgets.output_list,set_value=''

state.values.selection_values=ptr_new(selection_values)
state.values.selected_values=ptr_new()

; Restore the state.
WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY

END

;-----------------------------------------------------------------------------

FUNCTION pickvalues_get_value,id

ON_ERROR, 2

stash = WIDGET_INFO(id, /CHILD)
WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY

selected_values=(*state.values.selected_values)

; Restore the state.
WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY

; Return the value here.
RETURN,selected_values

END

;-----------------------------------------------------------------------------

FUNCTION pickvalues_event,ev

	; This routine handles all the events that happen in your
	; compound widget and if the events need to be passed along
	; this routine should return the new event.  If nobody needs
	; to know about the event that just occured, this routine 
	; can just return 0.  If your routine never needs to pass
	; along an event, this routine can be a procedure instead
	; of a function.  Whichever type used must be set below in the
	; WIDGET_BASE call using either the EVENT_PROC or EVENT_FUNC 
	; keyword.  An event function that returns a scalar 0 is 
	; essentially an event procedure.

parent=ev.handler


; Retrieve the structure from the child that contains the sub ids.
stash = WIDGET_INFO(parent, /CHILD)
WIDGET_CONTROL, stash, GET_UVALUE=state, /NO_COPY
widget_control,ev.id,get_uvalue = eventval
CASE eventval OF
  'input_list' : BEGIN
    res=widget_info(ev.id,/list_select)
    selection=[(*state.values.selection_values)(res)]
    IF ptr_valid(state.values.selected_values) THEN BEGIN
      current_sel=(*state.values.selected_values)
      ind=where(current_sel EQ selection,count)
      IF count EQ 0 THEN BEGIN   ;add to the selection
        current_sel=[current_sel,selection]
      ENDIF
    ENDIF ELSE BEGIN
      current_sel=selection
    ENDELSE
    order=sort(current_sel)
    current_sel=current_sel(order)
    state.values.selected_values=ptr_new(current_sel)
    widget_control,state.widgets.output_list,set_value=current_sel
  END
  'output_list' : BEGIN        ;remove from selection
    res=widget_info(ev.id,/list_select)
    selected_values=(*state.values.selected_values)
    selection=selected_values(res)
    ind=where(selected_values NE selection AND selected_values NE '',count)
    IF count NE 0 THEN BEGIN
      selected_values=selected_values(ind)
    ENDIF ELSE BEGIN
      selected_values=['']
    ENDELSE
    order=sort(selected_values)
    selected_values=selected_values(order)
    widget_control,state.widgets.output_list,set_value=selected_values
    state.values.selected_values=ptr_new(selected_values)
  END
ENDCASE

WIDGET_CONTROL, stash, SET_UVALUE=state, /NO_COPY

;  RETURN, { ID:parent, TOP:ev.top, HANDLER:0L }
RETURN,0  ;does not have to propagate any further.
END

;-----------------------------------------------------------------------------

FUNCTION cw_pickvalues, parent, UVALUE = uval, value=value,already_selected=already_selected

	; You should not use the user value of the main base for
	; your compound widget as the person using your compound widget
	; may want it for his or her own use.  
	; You also should not use the user value of the first widget you
	; install in the base as it is used to keep track of the state.

; state structure for your compound widget.

IF (N_PARAMS() EQ 0) THEN MESSAGE, 'Must specify a parent for Cw_pickvalues'

ON_ERROR, 2					;return to caller

; Defaults for keywords
IF NOT (KEYWORD_SET(uval))  THEN uval = 0

w_state={input_list:0L,output_list:0L}
v_state= {id:0,selection_values:ptr_new(),selected_values:ptr_new()}
IF keyword_set(already_selected) THEN v_state.selected_values=ptr_new(already_selected)

mainbase = WIDGET_BASE(parent, UVALUE = uval, $
		EVENT_FUNC = "pickvalues_event", $
		FUNC_GET_VALUE = "pickvalues_get_value", $
		PRO_SET_VALUE = "pickvalues_set_value")
print,'mainbase:',mainbase

base = widget_base(mainbase,/row)
print,'base:',base
v_state.id=base
xs=max(strlen(value))
order=sort(value)
vvalue=value(order)
w_state.input_list=widget_list(v_state.id,value=vvalue,/multiple,ysize=10,xs=xs,uvalue='input_list')
w_state.output_list=widget_list(v_state.id,ysize=10,xs=xs,uvalue='output_list')
IF keyword_set(already_selected) THEN BEGIN
  widget_control,w_state.output_list,set_value=already_selected
ENDIF
v_state.selection_values=ptr_new(vvalue)

state={widgets:w_state,values:v_state}

WIDGET_CONTROL, base, SET_UVALUE=state, /NO_COPY

RETURN, mainbase

END





