FUNCTION my_CURVEFIT,X,Y,Weight,A,SIGMAA, fail,Function_Name = Function_Name, $
	 to_be_fit=to_be_fit,niter=niter,plot_it=plot_it,chisqr=chisqr, $
	 silent=silent,chi2=chi2

;+
; NAME:
;	my_curvefit
;
; PURPOSE:
;	Non-linear least squares fit to a function of an arbitrary 
;	number of parameters.  The function may be any non-linear 
;	function where the partial derivatives are known or can be 
;	approximated.
;
; CATEGORY:
;	E2 - Curve and Surface Fitting.
;
; CALLING SEQUENCE:
;	Result = my_curvefit(X, Y, W, A, SIGMAA,fail, FUNCTION_NAME = name
;		niter=,/plot_it,chisqr=,/silent,to_be_fit=)
;
; INPUTS:
;	X = row vector of independent variables.
;
;	Y = row vector of dependent variable, the same length as x.
;
;	W = row vector of weights, the same length as x and y.
;		For no weighting,
;		w(i) = 1.0.
;		For instrumental weighting,
;		w(i) = 1.0/y(i), etc.
;
;	A = vector, with as many elements as the number of terms, that 
;	    contains the initial estimate for each parameter.  If A is double-
;	    precision, calculations are performed in double precision, 
;	    otherwise they are performed in single precision.
;
; KEYWORDS:
;	function_name = Name of the function (actually, a procedure) to 
;	                fit. default "FUNCT".
;			It must be written as described under RESTRICTIONS.
;	niter = Max number of iterations. Default 20
;	chisqr = chisqr to be reached. Default 1E-3
;	to_be_fit = parameters to be optimized if to_be_fit(i)=1. Default all
;	plot_it = to plot the data, fit, weigth, ... and the fit in the params
;	silent = to make the routine silent
;
; OUTPUTS:
;	fail = 1 if failed, 0 if succedded (ie reached chisqr before Niter)
;	       Achieved values are returned even if fail=1
;	A =    vector of parameters containing fit.
;	result = The fit over X values
;
; OPTIONAL OUTPUT PARAMETERS:
;	Sigmaa =  Vector of standard deviations for the parameters in A.
;	
; COMMON BLOCKS:
;	NONE.
;
; SIDE EFFECTS:
;	None.
;
; RESTRICTIONS:
;	The function to be fit must be defined and called FUNCT,
;	unless the FUNCTION_NAME keyword is supplied.  This function,
;	(actually written as a procedure) must accept values of
;	X (the independent variable), and A (the fitted function's
;	parameter values), and return F (the function's value at
;	X), and PDER (a 2D array of partial derivatives).
;	For an example, see FUNCT in the IDL User's Libaray.
;	A call to FUNCT is entered as:
;	FUNCT, X, A, F, PDER
; where:
;	X = Vector of NPOINT independent variables, input.
;	A = Vector of NTERMS function parameters, input.
;	F = Vector of NPOINT values of function, y(i) = funct(x(i)), output.
;	PDER = Array, (NPOINT, NTERMS), of partial derivatives of funct.
;		PDER(I,J) = Derivative of function at ith point with
;		respect to jth parameter.  Optional output parameter.
;		PDER should not be calculated if the parameter is not
;		supplied in call.
;
; PROCEDURE:
;	Copied from "CURFIT", least squares fit to a non-linear
;	function, pages 237-239, Bevington, Data Reduction and Error
;	Analysis for the Physical Sciences.
;
;	"This method is the Gradient-expansion algorithm which
;	combines the best features of the gradient search with
;	the method of linearizing the fitting function."
;
;	Iterations are performed until the chi square changes by
;	only 0.01% or until Niter iterations have been performed.
;
;	The initial guess of the parameter values should be
;	as close to the actual values as possible or the solution
;	may not converge.
;
;	Undefined values (indef=-32768.) in Y are not taken into account
;
; MODIFICATION HISTORY:
;	Written, DMS, RSI, September, 1982.
;	Does not iterate if the first guess is good.  DMS, Oct, 1990.
;	Added CALL_PROCEDURE to make the function's name a parameter.
;		(Nov 1990)
;	12/14/92 - modified to reflect the changes in the 1991
;		   edition of Bevington (eq. II-27) (jiy-suggested by CreaSo)
;	Adapted by JPB, to output a failure parameter, input a goal chisqr,
;	restrict fit to certain parameters
;-

;	ON_ERROR,2		;RETURN TO CALLER IF ERROR

IF n_params(0) EQ 0 THEN BEGIN
  print,'Call:res=my_curvefit(x,y,w,coefs,sigma,fail) '
  print,'K words: [Function_Name=][Niter=][to_be_fit=[]][,/plot_it][chisqr=]'
  print,'K words: [/silent]'
  print,'coefs on input contains first guess'
  goto,sortie
ENDIF
indef=-32768.

chi=1e-3
IF keyword_set(chisqr) THEN chi=chisqr
IF not keyword_set(Niter) THEN Niter=20
IF not keyword_set(to_be_fit) THEN BEGIN
  to_be_fit=FIX(a)
  to_be_fit(*)=1
ENDIF
IF not keyword_set(function_name) THEN function_name = "FUNCT"
w=weight
ind=where(x EQ indef or y eq indef,count)
IF count NE 0 THEN w(ind)=0.

fail=0

A = 1.*A
ind_fit=where(to_be_fit EQ 1)
NTERMS = N_ELEMENTS(A(ind_fit))				;# OF PARAMS.
NFREE = (N_ELEMENTS(Y)<N_ELEMENTS(X))-NTERMS 		;Degs of freedom
;print,'Nfree=',nfree
IF NFREE LE 0 THEN STOP,'Curvefit - not enough data points.'
FLAMBDA = 0.001						;Initial lambda
DIAG = INDGEN(NTERMS)*(NTERMS+1) 			;subscpt of diag elts

FOR ITER = 1,Niter DO BEGIN	;Iteration loop
;  EVALUATE ALPHA AND BETA MATRICIES.
  IF not keyword_set(silent) THEN BEGIN
    print,'params=',A(ind_fit)
  ENDIF
  CALL_PROCEDURE, Function_name,X,A,YFIT,PDER		;conpute fuction at A.
  pder=pder(*,ind_fit)
  BETA = (Y-YFIT)*W # PDER
  ALPHA = TRANSPOSE(PDER) # (W # (FLTARR(NTERMS)+1)*PDER)
  CHISQ1 = TOTAL(W*(Y-YFIT)^2)/NFREE			;present CHI
;  print,'chisq1=',chisq1,'chi=',chi
;							If good, no more iterate
;  if chisq1 lt total(abs(y))/1e12/NFREE then begin
  if chisq1 lt chi then begin
    IF not keyword_set(silent) THEN print,'requested chisqr reached'
    sigmaa = fltarr(nterms)				;Return all 0's
    return, yfit
  endif
  lasta=a
  lastyfit=yfit
; INVERT MODIFIED CURVATURE MATRIX TO FIND NEW PARAMETERS.
  old_bb=A
  REPEAT BEGIN
    C = SQRT(ALPHA(DIAG) # ALPHA(DIAG))
    ARRAY = ALPHA/C
    ARRAY(DIAG) = ARRAY(DIAG)*(1.+FLAMBDA)
;    print,'Flambda=',Flambda
    ARRAY = INVERT(ARRAY)
    move=ARRAY/C # TRANSPOSE(BETA)
    B = A(ind_fit)+ move ;NEW PARAMS
    BB=A & BB(ind_fit)=B
;    stop
    IF not keyword_set(silent) THEN BEGIN
      print,'params=',BB(ind_fit)
    ENDIF
    indinf=wherenan(BB(ind_fit),countinf)
    IF countinf NE 0 THEN  BEGIN
      print,'found Inf'
      yfit=lastyfit
      a=lasta
      goto,fail
    ENDIF
    CALL_PROCEDURE, Function_name,X,BB,YFIT		;evaluate func (no pder)
    ind=wherenan(yfit,countnan)
    IF countnan NE 0 THEN  BEGIN
      print,'found NaN'
      yfit=lastyfit
      a=lasta
      goto,fail
    ENDIF
    CHISQR = TOTAL(W*(Y-YFIT)^2)/NFREE			;new CHISQR
    FLAMBDA = FLAMBDA*10.	;ASSUME FIT GOT WORSE
;    print,'chisqr=',chisqr,'chisq1=',chisq1
    lastyfit=yfit
    lasta=a
    ind=where(bb(ind_fit)-old_bb(ind_fit) NE 0,count_mv)
;    print,count_mv
    old_bb=bb
    ind=wherenan(bb(ind_fit),countnan)
  ENDREP UNTIL CHISQR LE CHISQ1 OR count_mv EQ 0 OR countnan NE 0
  FLAMBDA = FLAMBDA/100.				;decrease FLAMBDA
  A=BB							;save new params.
  IF not keyword_set(silent) THEN BEGIN
    PRINT,'ITERATION =',ITER,' ,CHISQR =',CHISQR
    print,'========================================================'
  ENDIF
  IF ((CHISQ1-CHISQR)/CHISQ1) LE .0001 THEN BEGIN
    fail=1
    GOTO,DONE	;Finished?
  ENDIF
  IF CHISQR LE chi THEN BEGIN
    fail=0
    GOTO,DONE			;Finished?
  ENDIF
ENDFOR							;iteration loop
;
fail:

IF not keyword_set(silent) THEN message, 'Failed to converge', /INFORMATIONAL
fail=3
;print,'iter niter=',iter,niter
IF iter EQ Niter+1 THEN fail=2
IF keyword_set(plot_it) THEN BEGIN
  ind=where(w ne 0)
  plot,x,y,yrange=[min(y(ind)),max(y(ind))],xrange=[min(x(ind)),max(x(ind))], $
  xstyle=1,ystyle=1
  oplot,x,yfit,color=100
ENDIF
RETURN,yfit

DONE:
SIGMAA = SQRT(ARRAY(DIAG)/ALPHA(DIAG)) ;RETURN SIGMA'S
IF keyword_set(plot_it) THEN BEGIN
  ind=where(w ne 0)
  plot,x,y,yrange=[min(y(ind)),max(y(ind))],xrange=[min(x(ind)),max(x(ind))], $
  xstyle=1,ystyle=1
  oplot,x,yfit,color=100
ENDIF
chi2 = chisqr         ; Return chi-squared (chi2 obsolete-still works)

RETURN,YFIT						;return result
sortie:
END
