FUNCTION healp_smooth, map, fwhm, cc,fits_file = fits_file,ordering=ordering,header=header,exth=exth, $
                      correct=correct,silent=silent

;+
; NAME:
;    healp_smooth
; CATEGORY:
;    Healpix additions of JPBlib
; PURPOSE:
;    smoothes an healpix map to the required resolution
; CALLING SEQUENCE:
;    outmap=healp_smooth(map, fwhm[,cc][,fits_file=][,ordering=][,header=][,exth=][,/correct][,/silent])
; INPUT:
;    map = Healpix map (If a cube is given, it is assumed to be IQU)
;    fwhm = FWHM of the Gaussian to be used to convolve the map (degrees)
; OUTPUT:
;    outmap = smoothed map
; OPTIONAL INPUT:
;    None
; INPUT KEYWORDS:
;    fits_file = if set, this file is used to store the smoothed map
;    ordering  = ordering scheme of the input map [default=same as
;    write_fits_map]
;    correct   = if set, compute correction factor (caution, not
;                the factor is not applied to the data)
;    silent    = if set, the smoothing F90 program remains silent
; OUTPUT KEYWORDS:
;    header   = fits header of the resulting map
;    exth     = extension header of the resulting map
; SIDE EFFECTS:
;    None
; EXAMPLES:
;    
; RESTRICTIONS:
;    works with the Fortran code SMOOTHING 1.2.0 which should be compiled
; PROCEDURES CALLED:
;    SMOOTHING 1.2.0
; REVISION HISTORY:
;    Created by Jean-Philippe Bernard 21/01/2004 Strongly inspired from heal_smooth.pro by N. Ponthieu
;-

IF n_params() LT 1 THEN BEGIN
   print, 'outmap=healp_smooth(map,fwhm[,cc][,fits_file=][,ordering=][,header=][,exth=][,/correct][,/silent])'
   print, '       cubes for map indicated IQU'
   print, '       fwhm in degrees'
   GOTO,closing
ENDIF

si=size(map)
IF si(0) EQ 2 THEN BEGIN
  polar='2'
ENDIF ELSE BEGIN
  polar='1'
ENDELSE

dir=filepath('',/tmp)   ;This is the directory where all will take place
inputfile=dir+'map2bsmoothed.fits'
outputfile=dir+'smoothed_map.fits'
paramfile=dir+'smoothing.par'
logfile=dir+'smooth_log.txt'
if keyword_set(fits_file) then outputfile = fits_file

nside = npix2nside( n_elements(map))

openw,unit,paramfile,/get_lun
printf,unit,polar                 ;1) a Temperature only map, 2) Temperature + polarisation maps
printf,unit,inputfile
printf,unit,''                    ;use the default for lmax
printf,unit,0                     ;0) a standard analysis
printf,unit,''                    ;name of file with precomputed p_lms
printf,unit,fwhm*60.              ;Gaussian beam FWHM in arcmin
printf,unit,''                    ;external file name containing a symmetric beam Legendre transform
printf,unit,outputfile            ;output file
printf,unit,''                    ;Do you want to use ring weights in the analysis (1=yes,2=no)
close,unit
free_lun,unit

;if necessary, compute correction factor
IF keyword_set(correct) THEN BEGIN
  map2smooth = map * 0.
  ang2pix_ring, nside, 45, 45, pix
  map2smooth(pix) = 1
  write_fits_map,inputfile, map2smooth,ordering=ordering
  IF not keyword_set(silent) THEN BEGIN
    spawn, 'smoothing < '+paramfile
  ENDIF ELSE BEGIN
    spawn, 'smoothing < '+paramfile+' >'+logfile
  ENDELSE
  loaddata, outputfile, 1, tut_sm
;    cc = sqrt( total(tut_sm^2))  ;why ?
  cc= total(tut_sm)
  print, 'corr: '+strtrim(cc,2)
ENDIF

spawn, 'chown g=+rwx '+logfile

undef_values=!indef
;undef_values=min(map2smooth)
spawn, 'rm -f '+outputfile
spawn, 'rm -f '+inputfile
map2smooth = map
;Set undefined values to 0. for the convolution
indindef = where(map2smooth EQ undef_values,countindef)
IF countindef NE 0 THEN map2smooth(indindef) = 0.
IF polar EQ '1' THEN BEGIN
  write_fits_map,inputfile,map2smooth,ordering=ordering
ENDIF ELSE BEGIN
  write_tqu,inputfile,map2smooth,ordering=ordering
ENDELSE
IF not keyword_set(silent) THEN BEGIN
  spawn, 'smoothing < '+paramfile
ENDIF ELSE BEGIN
  spawn, 'smoothing < '+paramfile+' >'+logfile
ENDELSE
read_fits_map,outputfile,outmap, header,exthdr

IF countindef NE 0 THEN BEGIN
  outmap(indindef)=!indef
ENDIF

spawn, 'rm -f '+outputfile
spawn, 'rm -f '+inputfile
spawn, 'rm -f '+paramfile
spawn, 'rm -f '+logfile

closing :

RETURN,outmap

END
