FUNCTION locate_issa,ra,dec,band=band,closest=closest,iris=iris

;+
; NAME:
;      locate_issa
; PURPOSE:
;      locate the nearest ISSA image from input ra, dec
; CATEGORY:
;      ISSA
; CALLING SEQUENCE:
;      files=locate_issa(ra,dec[,band=band][,closest=][,/iris])
; INPUTS:
;      ra,dec : ra,dec to be searched for (degrees)
; OPTIONAL INPUTS:
;      Band   = IRAS band for which the filenames have to be
;               returned (1=12 mic to 4=100 mic)
;      closest= number of files to be returned
;               (in order of increasing distance from ra,dec
; OUTPUTS:
;      files  = list of filenames
; OPTIONAL OUTPUTS:
;      None
; COMMON BLOCKS:
;      None
; SIDE EFFECTS:
;      Uses an Xcatscan-like file in !jpblib_issa_dir
; RESTRICTIONS:
;      Must have read permission
; PROCEDURE:
;      Uses
;      read_xcat.pro
;           gcirc.pro
; EXAMPLE:
;      ra=ten(16.,24,0)/24.*360. &
;           dec=ten(-10.,0.,0.)
;      file=locate_issa(ra,dec,band=4,closest=2)
;
;      d=readfits(file(0),h)
;      image_cont20,d,h,/square
; MODIFICATION HISTORY:
;      Created
;      Oct 30 1997 by J.Ph. Bernard IAS
;      Modified June 2 1998
;      for MacOS JPB
;      Modified Mai 08 2000 to adapt for possible different
;      wbchar
;      Modified Jan 13 2005 to add iris keyword
;-

IF N_PARAMS(0) LT 2 THEN BEGIN
  PRINT,'Calling Sequence: '
  PRINT,'files=locate_issa(ra,dec[,band=band][,closest=][,/iris])'
  PRINT,'   (coordinates in degrees)'
  GOTO, closing
ENDIF

dir=!jpblib_issa_dir

IF not keyword_set(IRIS) THEN BEGIN
  data_dir=!iras_data_dir
ENDIF ELSE BEGIN
  data_dir=!iris_data_dir
ENDELSE

CASE !version.os OF
  'hp-ux': BEGIN
      bchar='B'
      xcatfile='issa_all_pos_unix.xcat'
   END
  'OSF': BEGIN
      bchar='B'
      xcatfile='issa_all_pos_unix.xcat'
   END
   'MacOS': BEGIN
      bchar='B'
      xcatfile='issa_all_pos.xcat'
   END
   'darwin': BEGIN
     bchar='B'
     xcatfile='issa_all_pos.xcat'
   END
   'linux': BEGIN
      bchar='B'
      xcatfile='issa_all_pos_unix.xcat'
   END
   ELSE: BEGIN
      print,'Operating system '+!version.os+' not recognized by locate_issa.pro'
   END
ENDCASE

cat_name=xcatfile
st=read_xcat(dir+cat_name,/silent)
onefile=st(0).filenames
pos=strposmulti(onefile,'H0.',count)
IF count NE 0 THEN BEGIN
  wbchar=strmid(onefile,pos(count-1)-1,1)
ENDIF ELSE BEGIN
  message,'string H0. not found in first file name : '+onefile
ENDELSE

IF NOT keyword_set(closest) THEN BEGIN
  closest=1
ENDIF
IF NOT keyword_set(band) THEN band=1

st=read_xcat(dir+cat_name,/silent)

gcirc,0,ra/180.*!pi,dec/180.*!pi,st.crval1/180.*!pi,st.crval2/180.*!pi,dist

order=sort(dist)

;ind=where(dist EQ min(dist))

name=st(order(0:closest-1)).filenames

IF band NE 1 THEN BEGIN
  FOR i=0,closest-1 DO BEGIN
    strs=bchar+wbchar
    pos=strposmulti(name(i),strs,count)
    IF count EQ 0 THEN BEGIN
      message,'The string : '+strs+' Was not found in file name'
    ENDIF
    bband=bchar+strtrim(fix(band),2)
    name(i)=strmid(name(i),0,pos(0))+bband+ $
            strmid(name(i),pos(0)+2,strlen(name(i))-pos(0))
  ENDFOR
ENDIF

name=data_dir+name

RETURN,name
closing:

END

