FUNCTION mosaique_issa,hh,band,weight=weight,show=show,lbl=lbl,iris=iris

;+
; NAME:
;      mosaique_issa
; PURPOSE:
;      creates a mosaic of ISSA images
; CATEGORY:
;      ISSA
; CALLING SEQUENCE:
;      im=mosaique_issa(hh,band[,weight=][,/show][,/lbl][,/iris])
; INPUTS:
;      hh : header on which projection has to be done
;      band = IRAS band for which the filenames have to be
;               returned (1=12 mic to 4=100 mic)
; OPTIONAL INPUTS:
;      show = show intermediate images, if set.
;      lbl  = if set, do calculation line by line (saves memory but
;      less efficient)
; OUTPUTS:
;      im  = image corresponding to h
; OPTIONAL OUTPUTS:
;      weight = weight map for im
; COMMON BLOCKS:
;      None
; SIDE EFFECTS:
;      None
; RESTRICTIONS:
;      None
; PROCEDURE:
;      Uses locate_issa and mosaique3
;      recursively until all the image area has been covered.
;      Uses locate_issa.pro, mosaique3.pro,
;      create_coo2.pro, index2ij.pro
; EXAMPLE:
;
;      ra=ten(16.,24,0)/24.*360. & dec=ten(-20.,0.,0.)
;
;      file=locate_issa(ra,dec,band=4,closest=2)
;
;      d=readfits(file(0),h)
;
;      sxaddpar,h,'CDELT1',sxpar(h,'CDELT1')*2.
;      sxaddpar,h,'CDELT2',sxpar(h,'CDELT2')*2.
;      window,0,xs=500,ys=500 & loadct,13
;      im=mosaique_issa(h,4,weight=w,/show)
;      image_cont20,im,h,/square,imrange=[-100,300]
; MODIFICATION HISTORY:
;      Created Oct 30 1997 by J.Ph. Bernard IAS
;      Modified June 2 1998 for MacOS JPB
;      Modified April 12 2000 for Galactic & Ecliptic headers by JPB
;      Modified Mai 08 2000 for Galactic & Ecliptic headers by JPB
;      Modified July 21 2000 test on file name to avoid looping by JPB
;      Modified Jan 13 2005 by JPB to include iris keyword
;-

IF N_PARAMS(0) LT 2 THEN BEGIN
  print,'Calling Sequence:im=mosaique_issa(h,band[,weight=][,/show][,/lbl][,/iris])'
  goto,sortie
ENDIF

create_coo2,hh,coo1,coo2,/silent,lbl=lbl,/remove

naxis1=sxpar(hh,'NAXIS1')
naxis2=sxpar(hh,'NAXIS2')
ctyp=strmid(sxpar(hh,'CTYPE1'),0,4)
IF ctyp EQ 'GLON' THEN BEGIN
  euler,coo1,coo2,ra,dec,2
  precess_array,ra,dec,astron_default_equinox(),1950.
  coo1=ra & coo2=dec
ENDIF
IF ctyp EQ 'ELON' THEN BEGIN
  euler,coo1,coo2,ra,dec,4
  precess_array,ra,dec,astron_default_equinox(),1950.
  coo1=ra & coo2=dec
ENDIF

res=fltarr(naxis1,naxis2) & res(*,*)=!indef
weight=res & weight(*,*)=0
w=1
the_end=0
last_file=''
REPEAT BEGIN
  ind=where(weight EQ 0 AND coo1 NE !indef and coo2 NE !indef,count)
  IF count NE 0 THEN BEGIN
    xy=index2ij(ind,[naxis1,naxis2])
    c1=coo1(xy(count/2,0),xy(count/2,1))
    c2=coo2(xy(count/2,0),xy(count/2,1))
    file=locate_issa(c1,c2,band=band,iris=iris)
    file=file(0)
    IF file EQ last_file THEN BEGIN
;      stop
      the_end=1
      goto,fin
    ENDIF
try_again:    
    f=findfile(file,count=c)
    last_file=file
    IF c NE 0 THEN BEGIN
      d=readfits(file,h)
      indd=where(d EQ !indef,cc)
      IF cc NE 0 THEN d(indd)=-2.00000e+09
      print,min(d)
      mosaique3,coo1,coo2,d,h,w,res,weight,lbl=lbl
      IF keyword_set(show) THEN BEGIN
        ind=where(res NE !indef,count)
        IF count NE 0 THEN BEGIN
          try=res
          try(ind)=alog10(res(ind))
          tvscl,try>0
;          stop
        ENDIF
      ENDIF
    ENDIF ELSE BEGIN
      print,'The file '+file+' was not found.'
      stop
      goto,try_again
;      goto,sortie
    ENDELSE
  ENDIF ELSE BEGIN
    the_end=1
  ENDELSE
fin:
ENDREP UNTIL the_end


RETURN,res

sortie:

END

