pro my_ticlabels, minval, numtics, incr, ticlabs, RA=ra, DELTA = delta
;+
; NAME:
;	MY_TICLABELS
; PURPOSE:
;	Used to display images with right ascension and declination
;	axes.  This routine creates labels for already determined tic
;	marks (every other tic mark)
;
; CALLING SEQUENCE:
;	my_ticlabels, minval, numtics, incr, ticlabs, [ RA = ,DELTA = ]
;
; INPUTS:
;	minval  - minimum value for labels (degrees)
;	numtics - number of tic marks
;	incr    - increment in minutes for labels
;
; OUTPUTS:
;	ticlabs - array of charater string labels
;
; OPTIONAL INPUT KEYWORDS:
;	RA - if this keyword is set then the grid axis is assumed to be
;		a Right Ascension.   Otherwise a declination axis is assumed
;	DELTA - Scalar specifying spacing of labels.   The default is 
;		DELTA = 2 which means that a label is made or every other tic
;		mark.  Set DELTA=1 to create a label for every tic mark.
;
; NOTES:
;	The following note applies to users of IDL version 2.4.0 or before.
;	This bug mentioned was fixed in Version 3.0.0
;
;	If you are using the native Postscript fonts, you must first define
;	your font with DEVICE,/TIMES,FONT_INDEX=3 where /TIMES is replaced with
;	your font of choice.  The default is Helvetica and therefore does not
;	require the above DEVICE command.  This patch is necessary if you are
;	using IDL 2.4.0 or before, because the !X feature does not work
;	properly using the PostScript Driver in this version.
;
; PROCEDURES USED:
;	RADEC
;
; RESTRICTIONS:
;	Invalid for wide field (> 2 degree) images since it assumes that a 
;	fixed interval in Y (or X) corresponds to a fixed interval in Dec 
;	(or RA)
;
; REVISON HISTORY:
;	written by B. Pfarr, 4/15/87
;	Added DELTA keywrd for compatibility with IMCONTOUR W. Landsman Nov 1991
;	Added nicer hms and dms symbols when using native PS fonts Deutsch 11/92
;	Added Patch for bug in IDL <2.4.0 as explained in NOTES E. Deutsch 11/92
;-
 On_error,2
;                               convert min to hours, minutes, secs

if N_params() LT 4 then begin
  print,'Syntax - my_ticlabels, minval, numtics, incr, ticlabs, ' + $
                     '[ /RA  ,DELTA = ]'
  return
endif

ticlabs = strarr( numtics )

if minval LT 0 then neg = -1 else neg = 1
firstval = minval + neg*0.00014              ;Roundoff to nearest arc-second

if not keyword_set( DELTA ) then delta = 2

if keyword_set( RA ) then begin                  ;Define RA tic symbols
  radec, firstval, 0, minh, minm, mins, dum1, dum2, dum3 
  sd = '!Ah!N' & sm = '!Am!N'  & ss = '!As!N' 
  if (!d.name eq 'PS') and (!p.font eq 0) then begin    ;Postscript fonts?
    sd ='!Uh!N' & sm='!Um!N' & ss='!Us!N' 
  endif
endif else begin 
  radec, 0, firstval, dum1, dum2, dum3, minh, minm, mins
  sd = "!Ao!N" & sm = "'" &  ss = "''"
  if (!d.name eq 'PS') and (!p.font eq 0) then begin
  if ( !VERSION.RELEASE LT '2.4.e' ) then RtEF = '!3' else  $
                                                 RtEF = '!X'
    sd = '!9' + string(176b) + RtEF 
    sm = '!9' + string(162b) + RtEF
    ss = '!9' + string(178b) + RtEF
  endif
endelse

 if incr LT 0 then rnd = -0.008333 else rnd = 0.008333  ;(30" = 0.0083333deg)

if (abs(mins) GT 1) or (abs(incr) LT 1.0/DELTA)  then begin      ;Seconds

      inc = fix( incr*60.0000*DELTA + rnd)
      ticlabs(0) = string( minh, '(i3)') + sd + ' ' + string(minm,'(i2)') + $ 
                 sm + ' ' + string( fix(mins), '(i2)') + ss
      for i = 1,numtics-1 do begin
         mins = mins + neg*inc

         if ( ( mins GE 60) or (mins LT 0) ) then begin

            while ( mins GE 60 ) do begin
                mins = mins - 60
                minm = minm + 1
            endwhile

            while ( mins LT 0 ) do begin
                mins = mins + 60
                minm = minm - 1
             endwhile
    
            ticlabs(i) = string( minm, '(i3)' ) + sm + ' '+ $
                         string( fix(mins), '(i2)') + ss

         endif else ticlabs(i) = string( fix(mins), '(i2)' ) + ss

      endfor

endif else $
    if (abs(minm) gt 1) or (abs(incr) LT 60.0/DELTA) then begin ;MINUTES

      inc = fix(incr*DELTA)
      ticlabs(0) = string(minh,'(i3)')+ sd+ ' ' +string(minm,'(i2)') + sm

      for i = 1,numtics-1 do begin
         minm = minm + neg*inc

         if (minm ge 60) then begin
             minm = minm - 60
             minh = minh + neg
             ticlabs(i)= string(minh,'(i3)')+sd+ ' '+string(minm,'(i2)') +sm

         endif else if (minm lt 0) then begin

             minm = minm + 60
             minh = minh - neg
             ticlabs(i)= string(minh,'(i3)')+sd+' '+string((minm),'(i2)')+sm

         endif else ticlabs(i) = string(minm,'(i5)')

      endfor

endif else begin                        ;Hours/Degrees

      inc = fix(DELTA*incr/60.0)
;      ticlabs(0) = string(minh,'(i3)') + sd 
      ticlabs(0) = string(minh,'(i4)') + sd 
      for i = 1,numtics-1 do begin
          minh = minh + inc
;         ticlabs(i) = string( minh, '(i3)' ) + sd
         ticlabs(i) = string( minh, '(i4)' ) + sd
      endfor      

endelse

return    
end
