FUNCTION integrated_intensity,input,header, $
                              vrange=vrange,silent=silent,elapsed_time=elapsed_time, $
                              output_header=output_header

;+
; NAME:
;	integrated_intensity
; PURPOSE:
;     compute integrated intensity map in a given vlsr range
; CATEGORY:
;     Molecular data reduction
; CALLING SEQUENCE:
;	ii=integrated_intensity(input,header,[vrange=])
; INPUTS:
;	d  = data cube
;       h  = data header
; OPTIONAL INPUT PARAMETERS:
;    vrange = vlsr range to be used (default=full range)
; OUTPUTS:
;	ii        = integrated intensity map
; OPTIONAL OUTPUT PARAMETERS:
;	None
; ACCEPTED KEY-WORDS:
;	silent = if set, behaves silently
;       elapsed_time = elapsed time (sec)
;       output_header= header for the ii map
; COMMON BLOCKS:
;	None
; SIDE EFFECTS:
;       Does not work with default vrange
; RESTRICTIONS:
; PROCEDURE:
;       make_axis.pro
; EXAMPLES
;    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard Sep 1 1998
;       JPB 1 Dec 2002, added elapsed_time keyword
;       JPB 5 Dec 2002, added handling blank value, units (Kkm/s vs K)
;       JPB 5 Dec 2002, added output keyword
;-

IF N_PARAMS(0) EQ 0 THEN BEGIN
  print,'Calling Sequence: ii=integrated_intensity(input,h)'
  print,'Accepted Key-words: vrange=[],/silent,elapsed_time=elapsed_time'
  goto,sortie
ENDIF

s=systime(1)

blank=sxpar(header,'BLANK',count=count)
IF count EQ 0 THEN BEGIN
  blank=!indef ;default value
  IF not keyword_set(silent) THEN BEGIN
    message,'blank assumed to be ='+strtrim(blank,2),/info
  ENDIF
ENDIF ELSE BEGIN
  IF not keyword_set(silent) THEN BEGIN
    message,'FITS blank value ='+strtrim(blank,2),/info
  ENDIF
ENDELSE

;this is the bunit in fits file
bunit=strupcase(strcompress(strtrim(sxpar(header,'BUNIT',count=count),2),/remove))
IF count EQ 0 THEN BEGIN
  bunit='K' ;default value
  IF not keyword_set(silent) THEN BEGIN
    message,'bunit assumed to be ='+bunit,/info
  ENDIF
ENDIF

pos=strposmulti(bunit,'KM/S',count)
IF count NE 0 THEN BEGIN
  bbunit=strmid(bunit,0,pos(0))
ENDIF ELSE BEGIN
  bbunit=bunit
ENDELSE
cunit3=strupcase(strtrim(sxpar(header,'CUNIT3',count=count),2))
IF count EQ 0 THEN BEGIN
  cunit3='KM/S'
  IF not keyword_set(silent) THEN BEGIN
    message,'cunit3 assumed to be ='+cunit3,/info
  ENDIF
ENDIF

IF not keyword_set(silent) THEN BEGIN
  message,'FITS header units ='+bunit,/info
  message,'used units ='+bbunit,/info
  message,'ii units will be  ='+bbunit+cunit3,/info
ENDIF

naxis3=sxpar(header,'NAXIS3')
cdelt3=sxpar(header,'CDELT3')
crval3=sxpar(header,'CRVAL3')
crpix3=sxpar(header,'CRPIX3')
dv=abs(cdelt3)

make_axis,header,coo1,coo2,vel
IF NOT keyword_set(vrange) THEN BEGIN
;  make_axis,header,coo1,coo2,vel
  velmi=min(vel)-cdelt3/2.
  velma=max(vel)+cdelt3/2.
ENDIF ELSE BEGIN
  velmi=min(vrange)
  velma=max(vrange)
ENDELSE

velmi=velmi>(min(vel))
velma=velma<(max(vel))

imin=(velmi-crval3)/cdelt3+crpix3-1
imax=(velma-crval3)/cdelt3+crpix3-1
iset=[imin,imax]
imin=min(iset) & imax=max(iset)
all_i=findgen(naxis3)
all_dv=fltarr(naxis3)

ind1=where(all_i EQ round(imin),count1)
all_dv(ind1(0))=dv*(ind1(0)+0.5-imin)
ind2=where(all_i GT round(imin) and all_i LT round(imax),count2)
IF count2 NE 0 THEN all_dv(ind2)=dv
ind3=where(all_i EQ round(imax),count3)
all_dv(ind3(0))=dv*(imax-(ind3(0)-0.5))

ii=input(*,*,0)
ii(*,*)=blank
ind=where(all_dv NE 0.,count)
FOR i=0,count-1 DO BEGIN
;  stop
  chan=all_i(ind(i))
  dv=all_dv(ind(i))
  im=input(*,*,ind(i))
  IF bunit EQ 'KKM/S' THEN BEGIN
    ind_ok=where(im NE blank,count_ok)
    IF count_ok NE 0 THEN BEGIN
      im(ind_ok)=im(ind_ok)/cdelt3
    ENDIF
  ENDIF
  indd=where(ii NE blank and im NE blank,countt)
  IF countt NE 0 THEN ii(indd)=ii(indd)+im(indd)*dv
  indd=where(ii EQ blank and im NE blank,countt)
  IF countt NE 0 THEN ii(indd)=im(indd)*dv
  IF NOT keyword_set(silent) THEN BEGIN
    print,'Channel '+strtrim(chan,2),' dv=',dv
  ENDIF  
ENDFOR
dvtot=total(all_dv)

IF NOT keyword_set(silent) THEN BEGIN
  print,'=== dvtot=',dvtot,'dvrange=',abs(velmi-velma)
ENDIF

;== compute fits header for output, if necessary
iibunit=bbunit+cunit3
output_header=header
sxdelpar,output_header,'CTYPE3'
sxdelpar,output_header,'CRVAL3'
sxdelpar,output_header,'CDELT3'
sxdelpar,output_header,'CUNIT3'
sxdelpar,output_header,'CRPIX3'
sxdelpar,output_header,'CRPIX3'
sxdelpar,output_header,'NAXIS3'
sxdelpar,output_header,'DATAMIN'
sxdelpar,output_header,'DATAMAX'
sxaddpar,output_header,'NAXIS',2,'Number of axis'
sxaddpar,output_header,'BUNIT',iibunit,'Integrated Intensity'
sxaddpar,output_header,'VELMI',velmi,'Min vlsr of integration ('+CUNIT3+')'
sxaddpar,output_header,'VELMA',velma,'Max vlsr of integration ('+CUNIT3+')'

elapsed_time=systime(1)-s
RETURN,ii

sortie:

END
