FUNCTION make_pos_vel_map,d,h,vrange,vstep,Ninter,angle,dl, $
                          cent_coord=cent_coord,vval=vval,xval=xval, $
                          xrange=xrange,dist=dist,stdmap=stdmap,silent=silent
;+
; NAME:
;    make_pos_vel_map
; PURPOSE:
;    Construct a position-velocity diagram from a cube of molecular data
;    from a center position along a given direction on the sky.
; CATEGORY:
;    image processing
; CALLING SEQUENCE: 
;	map=make_pos_vel_map(d,h,vrange,vstep,Ninter,angle,dl, $
;                          cent_coord=cent_coord,vval=vval,xval=xval, $
;                          xrange=xrange,dist=dist)
; INPUTS:
;	d,h    = data array (3D) and fits header
;	vrange = range of vlsr to be covered by map
;     vstep = step in vlsr of the map
;     Ninter = number of intervals for the integration along the cut
;              and the y-size of map
;     angle  = angle of the cut (degree) from North
;     dl     = width of the cut (degree)
; OPTIONAL INPUT PARAMETERS:
;     cent_coord = coordinates (degree) from the center (e.g. [ra,dec])
;                  default=center of the cube
;     xrange     = range of position values for map.
;                  default=all values
;     silent     = if set behaves silently
; OUTPUTS:
;	map     = position-velocity map. x and y axis correspond to velocity and
;              position respectively.
; OPTIONAL OUTPUT PARAMETERS:
;     vval = vlsr values (1D) corresponding to map
;     xval = position values (1D) corresponding to map
;     dist = distance array used in the averaging.
;   stdmap = standard deviation map (1-sig)
; COMMON BLOCKS:
;     None
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
;     Uses average_cut.pro, integrated_intensity.pro
; MODIFICATION HISTORY:
;      20-Aug-1998, Written by Jean-Philippe Bernard
;-

input_angle=angle

Nx=(size(d))(1) & Ny=(size(d))(2) & Nv=(size(d))(3)

make_axis,h,ra,dec,vel
Nvel=fix((Vrange(1)-vrange(0))/vstep)

posvmap=fltarr(Nvel,Ninter-1)
stdmap=posvmap				;standard deviation map (1-sig)
vval=fltarr(Nvel)

dummy=d(*,*,0)
;=== Make an index for bad values in the distance
dist=1
try=average_cut(dummy,h,angle=input_angle+90.,dist=dist,Ninter=Ninter,/silent, $
                  xval=xval,dval=dval)
create_coo2,h,coo1,coo2,/silent

IF keyword_set(cent_coord) THEN BEGIN
  ra_c=cent_coord(0)
  de_c=cent_coord(1)
ENDIF ELSE BEGIN
  ra_c=coo1(Nx/2,Ny/2)
  de_c=coo2(Nx/2,Ny/2)
ENDELSE

gcirc,0,ra_c/!radeg,de_c/!radeg,coo1/!radeg,coo2/!radeg,dd
ind=where(dd EQ min(dd)) & ij=index2ij(ind,[Nx,Ny]) & is=ij(0,0) & js=ij(0,1)
dsou=dist(is,js)
dist=abs(dist-dsou)
dlim=dl/2.
ind=where(dist GT dlim,count)
ii=dummy
ii(ind)=!indef
tvscl,ii>(-10)
try=average_cut(dummy,h,angle=input_angle,dist=dist,Ninter=Ninter,/silent, $
                  xval=xval,dval=dval)
dist=dist-dist(is,js)
dist(ind)=!indef
tvscl,abs(dist*60.*60)<20,Nx,0
;stop

FOR i=0,Nvel-1 DO BEGIN
  vr=[vrange(0)+i*vstep,vrange(0)+(i+1)*vstep]
  vval(i)=vr(0)+vstep/2.
  print,i,vr(0),vr(1),vval(i)
;  v=vel_map(d,h,ii,vrange=vr)
;  IF i EQ 108 THEN stop
  ii=integrated_intensity(d,h,vrange=vr,silent=silent)
  ii(where(dist EQ !indef))=!indef
  tvscl,ii>0
  try=average_cut(ii,h,angle=angle,Ninter=Ninter,/silent, $
                  xval=xval,dval=dval,input_dist=dist,std=std)
  posvmap(i,*)=try
  stdmap(i,*)=std
ENDFOR

IF keyword_set(xrange) THEN BEGIN
  ind=where(xval GE xrange(0) AND xval LE xrange(1),count)
  xval=xval(ind)
  posvmap=posvmap(*,ind)
  stdmap=stdmap(*,ind)
ENDIF

RETURN,posvmap
END
