PRO plot_chanel_map,d,h,vrange,Nx=Nx,Ny=Ny, $
                    common_header=common_header,fixed_scale=fixed_scale,log_scale=log_scale, $
                    rebin_factor=rebin_factor,degrade_resolution=degrade_resolution, $
                    levels=levels,overlay=overlay, $
                    postcript=postcript,ps_color=ps_color

;+
; NAME:
;     plot_chanel_map
; PURPOSE:
;     compute and plots channel maps
; CATEGORY:
;     Molecular data reduction
; CALLING SEQUENCE:
;     plot_chanel_map,d,h,vrange[,Nx=][,Ny=][,common_header=][,fixed_scale=][,log_scale=]
;                               [,rebin_factor=][,degrade_resolution=][,levels=][overlay=]
;                               [,postcript=][,ps_color=]
; INPUTS:
;     d              = data cube
;     h              = data header
; OPTIONAL INPUT PARAMETERS:
; Nx                 = Number of maps in the x direction (default=4)
; Ny                 = Number of maps in the y direction (default=4)
; common_header      = If set, data is reprojected on this header
; fixed scale        = If set, this scale [min,max] is used for maps.
; log_scale          = If set, a log scale is used for all maps.
; rebin_factor       = If set, this rebin factor is applied to all maps.
; degrade_resolution = If set, the angular resolution of each map is degraded by this (degree)
; levels             = set of contour values to be overplotted as contours on each maps
; overlay            = overlay (see create_overlay_st.pro) to be overlaid on each maps
; postrcipt          = If set, output is sent to this postcript file
; ps_color           = If set, postcript will be in color
; OUTPUTS:
;     None
; OPTIONAL OUTPUT PARAMETERS:
;     None
; ACCEPTED KEY-WORDS:
;     None
; COMMON BLOCKS:
;     None
; SIDE EFFECTS:
;     None
; RESTRICTIONS:
; PROCEDURE:
;     
; EXAMPLES
;    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard 11/04/2001
;-

IF N_PARAMS(0) EQ 0 THEN BEGIN
    print,'Calling Sequence:'
    print,'plot_chanel_map,d,h,vrange[,Nx=][,Ny=][,common_header=][,fixed_scale=][,log_scale=]'
    print,'                          [,rebin_factor=][,degrade_resolution=][,levels=][overlay=]'
    print,'                          [,postcript=][,ps_color=]'
    goto,sortie
ENDIF

sf=1.2
frmt='(F6.1)'
sample=1
remove_border=0
remove_indef=0

IF keyword_set(degrade_resolution) THEN BEGIN
  enlarge_size=[degrade_resolution,degrade_resolution]
ENDIF

IF not keyword_set(Nx) THEN NNx=4 ELSE NNx=Nx
IF not keyword_set(Ny) THEN NNy=4 ELSE NNy=Ny
Nmap=NNx*NNy
dV=(vrange(1)-vrange(0))/Nmap
IF NOT keyword_set(levels) THEN BEGIN
  lev1=0
ENDIF ELSE BEGIN
  lev1=levels
ENDELSE

FOR j=0,NNy-1 DO BEGIN
  FOR i=0,NNx-1 DO BEGIN
    iv=j*NNx+i
    vr=[vrange(0)+iv*dv,vrange(0)+(iv+1)*dV]
    ii=integrated_intensity(d,h,vrange=vr,/silent)
    hh=h
    IF keyword_set(degrade_resolution) THEN BEGIN
      ii=enlarge_image(ii,hh,enlarge_size,hout)
      hh=hout
    ENDIF
;    stop
;    IF i EQ 0 and j EQ 2 THEN stop
    IF keyword_set(rebin_factor) THEN BEGIN
      ii=rebin_image(ii,hh,hout,rebin_factor,sample=sample)
      hh=hout
    ENDIF
    IF keyword_set(degrade_resolution) THEN BEGIN
      ii=degrade_res(ii,hh,degrade_resolution,hout, $
                      remove_indef=remove_indef,remove_border=remove_border,indef=!indef)
      hh=hout
    ENDIF
    hh2=1 & ii=cut_image(ii,hh,hout=hh2,/silent)
    hh=hh2    
    IF keyword_set(common_header) THEN BEGIN
      ii=project2(hh,ii,common_header,/silent,/round)
    ENDIF
    IF iv EQ 0 THEN all_ii=fltarr((size(ii))(1),(size(ii))(2),NNx*NNy)
    all_ii(*,*,iv)=ii
  ENDFOR
ENDFOR

IF keyword_set(common_header) THEN BEGIN
  hdisplay=common_header
ENDIF ELSE BEGIN
  hdisplay=hh
ENDELSE

ind=where(all_ii NE !indef) & minv=min(all_ii(ind))

FOR j=0,NNy-1 DO BEGIN
  FOR i=0,NNx-1 DO BEGIN
    iv=j*NNx+i
    ii=all_ii(*,*,iv)
    vr=[vrange(0)+iv*dv,vrange(0)+(iv+1)*dV]
    overlay.set0.label(1)='!6'+strtrim(string(vr(0),format=frmt),2)+'<v<'+ $
                       strtrim(string(vr(1),format=frmt),2)
    is=(1.*i)/(1.*NNx) & js=(1.*j)/(1.*NNy) & ie=(1.*(i+1))/(1.*NNx) & je=(1.*(j+1))/(1.*NNy)
    noclose=1
    IF i EQ 0 AND j EQ 0 THEN BEGIN
      cleanplot & erase
      xticks=6 & yticks=7
      no_x_label=0 & no_y_label=0
    ENDIF ELSE BEGIN
      !x.ticks=xticks & !y.ticks=yticks
      !x.tickname(*)=' ' & !y.tickname(*)=' '
      no_x_label=1 & no_y_label=1
    ENDELSE
    IF ie-is GT je-js THEN BEGIN
      !p.position=[is,js,is+(je-js)*sf,je]
    ENDIF ELSE BEGIN
      !p.position=[is,js,ie,js+(ie-is)*sf]
    ENDELSE
    IF i EQ NNx-1 and j EQ NNy-1 THEN noclose=0
    levv1=lev1
    IF keyword_set(log_scale) THEN BEGIN
      ind=where(ii NE !indef)
      ii(ind)=alog10(ii(ind)-minv+1e-5)
      levv1=alog10(lev1-minv+1e-5)
    ENDIF
    scale=[-1,max(ii)]
    IF keyword_set(fixed_scale) THEN scale=fixed_scale
    IF keyword_set(levels) THEN BEGIN      
      IF i EQ 0 AND j EQ 0 THEN contour=create_contour_st(ii,lev1=levv1,/silent)
      contour.set0.image=ii
    ENDIF
    image_cont20,ii,hdisplay,imrange=scale,/square,/silent,/noerase, $
                 noclose=noclose,no_x_label=no_x_label, $
                 no_y_label=no_y_label,levels=contour, $
                 overlay=overlay, /nologo, $
                 off_bar_pos=off_bar_pos, bar_title=bar_title,bar_charsize=!p.charsize, $
                 postcript=postcript,ps_color=ps_color
  ENDFOR
ENDFOR

sortie:

END
