FUNCTION average_cut,map,h,angle=angle,Ninter=Ninter,dist=dist,xval=xval, $
                     silent=silent,dval=dval,input_distance=input_distance, $
                     median=median,std=std

;+
; NAME:
;    average_cut
; PURPOSE:
;    average a map along a given direction.
; CATEGORY:
;    image processing
; CALLING SEQUENCE: 
;	cut=average_cut(d,h,angle=angle,Ninter=Ninter,dist=dist,xval=xval, $
;                     silent=silent,dval=dval,input_distance=input_distance, $
;                     median=median,std=std
; INPUTS:
;	d,h    = data array (2D) and fits header
; OPTIONAL INPUT PARAMETERS:
;     Ninter = number of intervals for the integration along the cut (default=100)
;     angle  = angle of the cut (degree) from North (default=1st diagonal)
;     input_distance = a (2D) array of distance to be used in the averaging
;     silent     = if set, behave silently.
;     median   = if set, the median value is used instead of the average.
; OUTPUTS:
;	cut   = values averaged along the cut (1D) size given by Ninter
; OPTIONAL OUTPUT PARAMETERS:
;     xval = average (or median) position values (1D) along the cut (degrees).
;     dval = position values (1D) used to average into (degrees).
;              same as xval if no !indef values are excluded.
;     dist = distance array used in the averaging.
;      std = standard deviation values (1-sig)
; COMMON BLOCKS:
;     None
; SIDE EFFECTS:
; RESTRICTIONS:
; PROCEDURE:
;     !indef values in the image (or distance array, if input_distance is used) are
;        excluded from the processing.
;     Uses sxpar.pro, la_median.pro, la_mean.pro
; MODIFICATION HISTORY:
;      20-Aug-1998, Written by Jean-Philippe Bernard
;-

cd=ABS(sxpar(h,'CDELT1'))
si=size(map) & Nx=si(1) & Ny=si(2)
Nmax=max([Nx,Ny])
;=== make the distance array
IF not keyword_set(input_distance) THEN BEGIN
  dist=fltarr(3.*Nmax,3.*Nmax)
  FOR i=0,3.*Nmax-1 DO BEGIN
    dist(*,i)=i*cd
  ENDFOR
  IF not keyword_set(angle) THEN angle=-atan((1.*Nx)/(1.*Ny))*!radeg
  dist=rot(dist,angle,1.,3.*Nmax/2.,3.*Nmax/2.)
  dist=dist(Nx:2.*Nx-1,Ny:2.*Ny-1)
  ind=where(map EQ !indef,count)
  IF count NE 0 THEN dist(ind)=!indef
ENDIF ELSE BEGIN
  dist=input_distance
ENDELSE
IF not keyword_set(Ninter) THEN Ninter=100

indg=where(dist NE !indef)
mid=min(dist(indg)) & mad=max(dist(indg))
dval=findgen(Ninter)/(1.*Ninter-1)*(mad-mid)+mid
dval=dval(0:Ninter-1) & Nint=Ninter-1
;dv=(dval(1)-dval(0))/2.
dv=la_median(abs(dval-shift(dval,1)))/2.
dval=dval+dv
val=fltarr(Nint) & val(*)=!indef
dd=fltarr(Nint)
std=fltarr(Nint)
;stop
FOR i=0,Nint-1 DO BEGIN
  ind=where(dist GT dval(i)-dv and dist LE dval(i)+dv,count)
  IF not keyword_set(silent) THEN BEGIN
    print,i,dval(i)-dv,dval(i)+dv,count
  ENDIF
  IF count NE 0 THEN BEGIN
    st=stdev(map(ind))
    IF keyword_set(median) THEN BEGIN
      val(i)=la_median(map(ind))
      dd(i)=la_median(dist(ind))
    ENDIF ELSE BEGIN
      val(i)=la_mean(map(ind))
      dd(i)=la_mean(dist(ind))
    ENDELSE
    IF count GE 2 THEN BEGIN
      std(i)=stdev(map(ind))
    ENDIF ELSE BEGIN
      std(i)=!indef
    ENDELSE   
  ENDIF
ENDFOR

xval=dd
RETURN,val

END
