FUNCTION degrade_res,input,h,reso,hout, $
                     remove_border=remove_border,remove_indef=remove_indef, $
                     indef=indef,poly_coef=poly_coef, $
                     get_kernel=get_kernel,no_convol=no_convol,Nker=Nker
;+
; NAME:
;       degrade_res
; PURPOSE:
;       degrade the angular resolution of an image by convolving it
;       with a 2D gaussian.
;	   indef values in the original image are not considered
;	   if remove_border is set, values affected by the map edge
;       (closer than reso from the edge) are set to indef
; CATEGORY:
;       image_processing
; CALLING SEQUENCE:
;       result=degrade_res(input,h,reso,hout
;                   [,/remove_border][,/remove_indef][,indef=]
;                   [,get_kernel=][,/no_convol])      
; INPUTS:
;       input,h        = input image (2D or 3D) and header
;       reso           = angular resolution requiered (degree)
;       hout           = output header
; OPTIONAL INPUTS:
;       remove_border  = if set the edge of the map is set to indef
;       remove_indef   = if set the edge of indef zones is set to
;       indef
;	   indef          = specify indef value to be used
;       [default=!indef]
;       no_convol      = if set no
;       convolution is performed
; KEYWORD PARAMETERS:
; OUTPUTS:
;       result of the function = smoothed image
; OPTIONAL OUTPUTS:
;
;       get_kernel     = if set, the kernel is returned
; COMMON
;       BLOCKS:
;       None
; SIDE EFFECTS:
;       The final image does not really have the resolution reso !
;
;       RESTRICTIONS:
; PROCEDURE:
; EXAMPLE:
; MODIFICATION HISTORY:
;       written JPB 09-08-92
;-

IF N_PARAMS(0) EQ 0 THEN begin
    print,'Calling Sequence: im=degrade_res(input,h,reso)'
    print,'Accepted Key-words: /remove_border,/remove_indef,indef='
    print,'                    get_kernel=get_kernel,/no_convol'
    goto,sortie
endif

IF not keyword_set(indef) THEN BEGIN
;    blank=sxpar(h,'BLANK')
;    bscale=sxpar(h,'BSCALE')
;
;    bzero=sxpar(h,'BZERO')
;    indef=double(blank*bscale+bzero)

;    indef=!indef
ENDIF
;print,'indef=',indef

si=size(input)
IF si(0) LT 2 OR si(0) GT 3 THEN BEGIN
    print,'Input should be 2 or 3 D'
;goto,sortie
ENDIF
Nx=si(1)
Ny=si(2)
IF si(0) EQ 3 THEN Nz=si(3)

old_reso=ABS(sxpar(h,'CDELT1'))
IF reso LT old_reso THEN BEGIN
    print,'reso should be larger than input pixel size'
    goto,sortie
ENDIF

;define the Kernel for the convolution
IF not keyword_set(Nker) THEN Nker=FIX(reso/old_reso)
;print,'Nker=',Nker
dis=fltarr(Nker,Nker)
set_dist2,dis,fix(Nker/2),fix(Nker/2)
dis=sqrt(dis)*old_reso          ;in deg from center
IF keyword_set(poly_coef) THEN BEGIN
    kernel=poly(dis,poly_coef)
ENDIF ELSE BEGIN
    a=reso^2/4./alog(2.)        ;?
    kernel=exp(-dis^2./a)
ENDELSE

ind=where(kernel lt 1e-30,count)
IF count NE 0 THEN kernel(ind)=0.
tot=total(kernel)
kernel=kernel/tot

;Do the actual convolution
IF not keyword_set(no_convol) THEN BEGIN
    IF si(0) EQ 2 THEN BEGIN
        toto=input
        ind=where(toto EQ indef,count)
        IF count NE 0 THEN toto(ind)=0.
        res=convol(toto,kernel,/center)
        ind=where(input EQ indef,count)
        IF keyword_set(remove_indef) and count NE 0 THEN BEGIN
            ind2=enlarge(ind,Nker,Nx,Ny)
            res(ind2)=indef
        ENDIF
;    print,'Nker=',Nker
        IF keyword_set(remove_border) THEN BEGIN
            res(0:Nker-1,*)=indef
            res(si(1)-Nker:si(1)-1,*)=indef
            res(*,0:Nker-1)=indef
            res(*,si(2)-Nker:si(2)-1,*)=indef
        ENDIF
                                ;put back indef values
        IF count NE 0 THEN res(ind)=indef
    ENDIF ELSE BEGIN
        res=fltarr(Nx,Ny,Nz)
        FOR k=0,Nz-1 DO BEGIN
            print,'plan #',k
            im=reform(input(*,*,k))
            ind=where(im EQ indef,count)
            IF count NE 0 THEN im(ind)=0.
            imres=convol(im,kernel,/center)
            IF keyword_set(remove_border) AND count NE 0 THEN BEGIN
                ind2=enlarge(ind,Nker,Nx,Ny)
                imres(ind2)=indef
            ENDIF
            IF keyword_set(remove_border) THEN BEGIN
                imres(0:Nker-1,*)=indef
                imres(si(1)-Nker:si(1)-1,*)=indef
                imres(*,0:Nker-1)=indef
                imres(*,si(2)-Nker:si(2)-1)=indef
            ENDIF
            IF count NE 0 THEN imres(ind)=indef
            res(*,*,k)=imres
        ENDFOR
    ENDELSE
;    hout=write_header(h,'COMMENT','ang. resol. degraded
;    to'+string(reso))
;    hout=write_header(hout,'COMMENT','using degrade_res.pro')
    hout=h
    sxaddpar,hout,'COMMENT','ang. resol. degraded to'+string(reso)
    sxaddpar,hout,'COMMENT','using degrade_res.pro'
ENDIF ELSE BEGIN
    res=!indef
ENDELSE

IF keyword_set(get_kernel) THEN get_kernel=kernel

RETURN,res
sortie:

END

