FUNCTION fits2healp_jpb,fits_image,fits_header,healp_header,elapsed_time=elapsed_time, $
                        weight=weight,fwhm=fwhm,kernel_radius=kernel_radius,phi=phi,theta=theta, $
                        interp_ptr=interp_ptr,use_interp_ptr=use_interp_ptr, $
                        healp_indexes=healp_indexes,use_healp_indexes=use_healp_indexes, $
                        lon=lon,lat=lat,silent=silent,nearest=nearest

;+
; NAME:
;	fits2healp_jpb
; PURPOSE:
;       reproject fits image onto a given healpix header (map or cube)
; CATEGORY:
;       Planck Ancillary data ingestion
; CALLING SEQUENCE:
;	res=fits2healp_jpb(fits_image,fits_header,healp_header)
; INPUTS:
;	fits_image  = fits image (2D or 3D)
;	fits_header = fits header
;	healp_header= healpix header
; OPTIONAL INPUT PARAMETERS:
; OUTPUTS:
; OPTIONAL OUTPUT PARAMETERS:
;	theta, phi    = native spherical coordinates
; ACCEPTED KEY-WORDS:
;	elapsed_time  = elapsed time in the routine (sec)
;       weight        = output weight vector
;	fwhm          = FWHM for Gaussian convolution (degree)
;	kernel_radius = radius of Gaussian kernel (degree)
;       phi           = first native coordiante of fits file (used to compute distances)
;       theta         = seconde native coordiante of fits file (used to compute distances)
;       interp_ptr    = outputs a pointer towards interpolation information
;       use_interp_ptr= use provided pointer towards interpolation information
;       healp_indexes = outputs a pointer toward healpix indices involved in projection
;       use_healp_indexes= use provided pointer toward healpix indices involved in projection
;       lon           = longitude of healpix data points (degree)
;       lat           = latitude of healpix data points (degree)
; COMMON BLOCKS:
;	None
; SIDE EFFECTS:
;       None
; RESTRICTIONS:
; PROCEDURE:
;	
; EXAMPLES
;    
; MODIFICATION HISTORY:
;	Written by J.P. Bernard 14/12/02
;       Modified JPB 9 Avril 2003: Added dealing with undefined values
;                                  in the convolution
;-


;---------------------------------------------------------------
; parameter check
;---------------------------------------------------------------
IF N_PARAMS(0) LT 3 THEN BEGIN
  PRINT,'Calling Sequence: '
  PRINT,'hvec=fits2healp_jpb(fits_image,fits_header,healp_header'
  PRINT,'                 [,elapsed_time=][,weight=][,fwhm=][,kernel_radius=]'
  PRINT,'                 [,phi=][,theta=][lon=][lat=]'
  PRINT,'                 [,interp_ptr=][,use_interp_ptr=][,healp_indexes=][,use_healp_indexes=]'
  GOTO, the_end
ENDIF

time=systime(1)

within=0.  ;default
IF keyword_set(kernel_radius) THEN BEGIN
  within=kernel_radius
ENDIF ELSE BEGIN
  IF keyword_set(fwhm) THEN BEGIN
    within=10.*fwhm                  ;default kernel radius for Gaussian interpolation
  ENDIF
ENDELSE

IF keyword_set(fwhm) THEN BEGIN
  sigma=fwhm/2./sqrt(2.)/SQRT(alog(2.))
  sigma2=sigma^2
ENDIF

nside=sxpar(healp_header,'NSIDE')
ordering=strtrim(sxpar(healp_header,'ORDERING'),2)
Naxis1=sxpar(fits_header,'NAXIS1')
Naxis2=sxpar(fits_header,'NAXIS2')
cdelt1=sxpar(fits_header,'CDELT1')
cdelt2=sxpar(fits_header,'CDELT2')
crpix1=sxpar(fits_header,'CRPIX1')
crpix2=sxpar(fits_header,'CRPIX2')

Npix=12L*(1.*nside)^2

healp_vec=fltarr(Npix) & healp_vec(*)=!indef
weight=fltarr(Npix) & weight(*)=0.

;=== compute healpix coordinates if not provided
IF not keyword_set(lon) OR not keyword_set(lat) THEN BEGIN
  healp_coord,nside,lon,lat,ordering=ordering
ENDIF
;stop
;=== transform lon,lat into the coordinate system of the fits file
healpcoo2fits,lon,lat,healp_header,fits_header,llon,llat

h2pix,fits_header,ipix_array,jpix_array,/silent
;compute fits pixels corresponding to all healpix coordinates
coo2pix2,llon,llat,pixx,pixy,/silent
;== set to nearest neighbor, if requested
IF keyword_set(nearest) THEN BEGIN
  pixx=round(pixx)
  pixy=round(pixy)
ENDIF
;determines radius encercling fits image
;pix2coo2,crpix1,crpix2,c1_0,c2_0,/silent
pix2coo2,(1.*naxis1)/2.,(1.*naxis2)/2.,c1_0,c2_0,/silent
create_coo2,fits_header,coo1,coo2,/silent
gcirc,0,c1_0/!radeg,c2_0/!radeg,coo1/!radeg,coo2/!radeg,dist
Rmax=max(dist)*!radeg
;compute distance between all healpix coordinates and fits center
gcirc,0,c1_0/!radeg,c2_0/!radeg,llon/!radeg,llat/!radeg,dist
;Select healpix pixels that have a chance to fall in the fits map within the kernel radius
ind0=where(dist LE (Rmax+within)/!radeg,count0)
pixx=pixx(ind0) & pixy=pixy(ind0)
print,'within=',within,'Rmax=',Rmax,'count0=',count0

last_iperc=-1

IF NOT keyword_set(fwhm) THEN BEGIN
  inmap=pix_in_map(fits_header,pixx,pixy,within=within)
  ind=where(inmap EQ 1,N_healp_pixel)
  healp_indexes=ind0(ind)
  FOR i=0L,N_healp_pixel-1 DO BEGIN
    healp_vec(healp_indexes(i))=bilinear(fits_image,pixx(ind(i)),pixy(ind(i)))
  ENDFOR
  weight(healp_indexes)=1.
ENDIF ELSE BEGIN
  IF NOT keyword_set(use_interp_ptr) THEN BEGIN
    inmap=pix_in_map(fits_header,pixx,pixy,within=within)
;     stop
    ind=where(inmap EQ 1,N_healp_pixel)
    healp_indexes=ind0(ind)             ;index of healpix positions inside fits map in original healpix
    interp_ptr=ptrarr(N_healp_pixel,2)
    message,'Starting Gauss interpolation',/info
    print,'N_healp_pixels=',N_healp_pixel
;    h2pix,fits_header,/silent
    h2pix,fits_header,ii,jj,/silent
;    plot,ii,jj,psym=3
    s=systime(1)
    sttot=0.
    FOR i=0L,N_healp_pixel-1 DO BEGIN
      ;compute fits neighboor index of given healpix position
;      neighb_index=fits_neighbors(fits_header,fits_image,pixx(ind(i)),pixy(ind(i)), $
;                                   within,distance=distance, $
;                                   phi=phi,theta=theta,count=Nneighb, $
;                                   elapsed_time=elapsed_time)
      ;stop
      neighb_index=fits_neighbors2(fits_header,naxis1,naxis2,coo1,coo2, $
                                   ii,jj,pixx(ind(i)),pixy(ind(i)), $
			           within,distance=distance, $
				   count=Nneighb,elapsed_time=elapsed_time)
      sttot=sttot+elapsed_time
;      print,strtrim((systime(1)-s)/60.,2)+' '+strtrim(sttot/60.,2)
;      print,'Nneighb=',Nneighb,' dist max=',max(distance)*60.,' arcmin' 
;      help,neighb_index,distance           
      IF Nneighb NE 0 THEN BEGIN
;        oplot,ii(neighb_index),jj(neighb_index),psym=4,color=100
;        stop
        ww=exp(-distance*distance/2./sigma2)
;;       we=total(ww)
;;       healp_vec(healp_indexes(i))=total(fits_image(neighb_index)*ww)/we
;;;       healp_vec(healp_indexes(i))=avg(fits_image(neighb_index))
;;       weight(healp_indexes(i))=we

        indd=where(fits_image(neighb_index) NE !indef,ccount)
        IF ccount NE 0 THEN BEGIN
          www=ww(indd) & we=total(www)
          healp_vec(healp_indexes(i))=total(fits_image(neighb_index(indd))*www)/we
;          healp_vec(healp_indexes(i))=avg(fits_image(neighb_index))
          weight(healp_indexes(i))=we
        ENDIF ELSE BEGIN
	  healp_vec(healp_indexes(i))=!indef
	  weight(healp_indexes(i))=0.
        ENDELSE
        interp_ptr(i,0)=ptr_new(neighb_index)
        interp_ptr(i,1)=ptr_new(ww)
        iperc=round(1.*i/(N_healp_pixel-1)*100)
        IF (iperc mod 10) EQ 0 and not keyword_set(silent) AND $
            iperc NE last_iperc THEN BEGIN
          message,'Done '+strtrim(iperc)+' % ...',/info
          last_iperc=iperc
;          print,strtrim((systime(1)-s)/60.,2)+' '+strtrim(sttot/60.,2)
        ENDIF
      ENDIF
    ENDFOR
    message,'Ending Gauss interpolation'+strtrim((systime(1)-s)/60.,2)+' min',/info
    
  ENDIF ELSE BEGIN
    h2pix,fits_header,ii,jj,/silent
;    plot,ii,jj,psym=3
    N_healp_pixel=(size(use_interp_ptr))(1)
    healp_indexes=use_healp_indexes
    FOR i=0L,N_healp_pixel-1 DO BEGIN
      IF ptr_valid(use_interp_ptr(i,0)) THEN BEGIN
        ww=(*use_interp_ptr(i,1))
        neighb_index=(*use_interp_ptr(i,0))
;        oplot,ii(neighb_index),jj(neighb_index),psym=4,color=100
        indd=where(fits_image(neighb_index) NE !indef,ccount)
        IF ccount NE 0 THEN BEGIN
          www=ww(indd) & we=total(www)
          healp_vec(healp_indexes(i))=total(fits_image(neighb_index(indd))*www)/we
;          healp_vec(healp_indexes(i))=avg(fits_image(neighb_index))
          weight(healp_indexes(i))=we
        ENDIF ELSE BEGIN
	  healp_vec(healp_indexes(i))=!indef
	  weight(healp_indexes(i))=0.
        ENDELSE
      ENDIF
    ENDFOR
  ENDELSE
ENDELSE

ind=where(healp_vec EQ !indef,count)
IF count NE 0 THEN weight(ind)=0.

elapsed_time=(systime(1)-time)

RETURN,healp_vec
the_end:

END

