PRO flux_values2,im,h,center=center,title=title, $
                distance=distance,xrange=xrange,yrange=yrange,N_values=N_values, $
                prof_radius=prof_radius,avg_prof=avg_prof,err_prof=err_prof, $
                all_flux=all_flux,all_stflux=all_stflux,all_Nflux=all_Nflux, $
                all_back=all_back,all_stback=all_stback,all_Nback=all_Nback, $
                no_plot=no_plot,dist_max=dist_max,flux_radius=flux_radius, $
                log_plot=log_plot
;+
; NAME:
;        flux_values
; PURPOSE:
;        Computes background subtracted fluxes in boxes of increasing
;        size around a source
;        also computes average profiles (and error) around a source
; CATEGORY:
;        image_processing
; CALLING SEQUENCE:
;         flux_values,im,h, $
;                     center=center,db=db,circle=circle, $
;                     distance=distance,Npix_moy=Npix_moy, $
;                     avg_radius=avg_radius,avg_prof=avg_prof,err_prof=err_prof, $
;                     all_flux=all_flux,all_stflux=all_stflux,all_Nflux=all_Nflux, $
;                     all_back=all_back,all_stback=all_stback,all_Nback=all_Nback, $
;                     no_plot=no_plot,xrange=xrange,yrange=yrange,title=title
; INPUTS:
;         im,h   : image and header. h used only to convert fluxes to
;                  Jy assuming im is in MJy/sr
; OPTIONAL INPUTS:
;         center :  pixel position of the source [x0,y0].
;                   default = position of maximum in im
;         db     :  size of the region used for background subtraction
;                   width in pixels. Default=1
;	  circle :  if set, calculations are done in circles
;         distance :  if set, calculations are done along the distance
;                   provided. same size a im
;         Npix_moy: Number of pixels used to bin the average
;                   profile. default=1
;         title  : title for the plot
;         xrange : xrange for the plot
;         yrange : yrange for the plot
; KEYWORD PARAMETERS:
;         no_plot    : if set, no plot is done
; OUTPUTS:
; OPTIONAL OUTPUTS:
;         avg_radius: average radius (arcmin)
;         avg_profile: average profile
;         err_prof   : rms error on avg_profile (1-sigma)
;         all_flux   : Background subtracted fluxes in increasing size boxes
;         all_stflux : rms error on the above (1-sigma)
;         all_Nflux  : number of defined pixels in flux boxes
;         all_back   : Background values in region of width db
;                      surrounding each integration box
;         all_stback : error (1-sigma) on the above
;         all_Nback  : number of defined pixels in each background region
; COMMON BLOCKS:
;         None
; SIDE EFFECTS:
;         The size of the outputs is governed by Npix_moy and the size
;         of the image.
;         The image is assumed to be in MJy/sr so that fluxes are in Jy.
; RESTRICTIONS:
; PROCEDURE:
;         Uses sum_box and mean_box
; EXAMPLE:
; MODIFICATION HISTORY:
;         Created by J.Ph Bernard Aug 1 1997
;-

;Check parameters list
IF n_params(0) LT 2 THEN BEGIN
  print,'Calling Sequence:flux_values,im,h'
  print,'                 center=center,db=db,/circle,distance=distance'
  print,'                 avg_radius=avg_radius,avg_prof=avg_prof,err_prof=err_prof'
  print,'                 all_flux=all_flux,all_stflux=all_stflux,all_Nflux=all_Nflux'
  print,'                 all_back=all_back,all_stback=all_stback,all_Nback=all_Nback'
  print,'                 no_plot=no_plot,xrange=xrange,yrange=yrange,title=title'
  goto,sortie
ENDIF

;Computes flux values from an image inside boxes of increasing size
;uses sum_box.pro recursively

old_pmulti=!p.multi
IF not keyword_set(title) THEN title=' '

si=size(im) & Nx=si(1) & Ny=si(2)

IF not keyword_set(Npix_moy) THEN Npix_moy=1
IF not keyword_set(db) THEN db=1

;If center not set, peak the max of the image
IF not keyword_set(center) THEN BEGIN
    ind=where(im EQ max(im))
    coo=index2ij(ind,[Nx,Ny])
    x0 = coo(0,0) & y0=coo(0,1)
ENDIF ELSE BEGIN
    x0=center(0) & y0=center(1)
ENDELSE
;Define a distance array (in pixels), if necessary
IF NOT keyword_set(distance) THEN BEGIN
    distance=dblarr(Nx,Ny)
    set_dist2,distance,round(x0),round(y0)
    distance=sqrt(distance)
ENDIF
;Define number of values
IF not keyword_set(N_values) THEN BEGIN
  N_values=Nx
ENDIF

;Set the number of values in the outputs
Vmax=max(distance)                  ;in pixels
IF keyword_set(dist_max) THEN Vmax=dist_max

maxdist_vals=findgen(N_values)/(1.*(N_values-1))*Vmax ;in [0,dist_max]

all_Flux=fltarr(N_values)          ;Array of flux values
all_back=fltarr(N_values)       ;Array of Background values
all_Nflux=lonarr(N_values)      ;Array of number of pixels in flux region
all_Nback=lonarr(N_values)      ;Array of number of pixels in background region
all_size=fltarr(N_values)
all_stback=fltarr(N_values)     ;Array of stdev in background region
all_stflux=fltarr(N_values)     ;Array of stdev in flux region

avg_prof=fltarr(N_values)
err_prof=fltarr(N_values)
all_size_prof=fltarr(N_values)
all_Nprof=lonarr(N_values)

dist_back=median(maxdist_vals-shift(maxdist_vals,1))

FOR i=0,N_values-1 DO BEGIN
  ind_flux=where(distance LE maxdist_vals(i) AND $
                  im NE !indef,count_flux)
  ind_back=where(distance GT maxdist_vals(i) AND $
                 im NE !indef AND $
                 distance LE maxdist_vals(i)+dist_back,count_back)
  all_Nflux(i)=count_flux
  all_Nback(i)=count_back
  IF count_back NE 0 THEN BEGIN
    stdd=moment(im(ind_back))
    all_back(i)=stdd(0)
    all_stback(i)=sqrt(stdd(1))
  ENDIF ELSE BEGIN
    all_back(i)=0. & all_stback(i)=!indef
  ENDELSE
  all_flux(i)=total(im(ind_flux))
  IF all_back(i) NE 0 THEN BEGIN
    all_flux(i)=all_flux(i)-all_back(i)*count_flux
    all_stflux(i)=all_stback(i)/sqrt(count_back)*count_flux
  ENDIF ELSE BEGIN
    all_stflux(i)=!indef
  ENDELSE
  all_size(i)=maxdist_vals(i)

  ind_prof=where(distance GE maxdist_vals(i)-dist_back/2. and $
                 distance LT maxdist_vals(i)+dist_back/2. and $
                 im NE !indef,count_prof)
  all_Nprof=count_prof
  IF count_prof GE 2 THEN BEGIN
    stdd=moment(im(ind_prof))
    avg_prof(i)=stdd(0)
    err_prof(i)=stdd(1)
    all_size_prof(i)=avg(distance(ind_prof))
  ENDIF ELSE BEGIN
    IF count_prof EQ 1 THEN BEGIN
      avg_prof(i)=im(ind_prof)
      err_prof(i)=!indef
      all_size_prof(i)=avg(distance(ind_prof))
    ENDIF ELSE BEGIN
      avg_prof(i)=!indef
      err_prof(i)=!indef
      all_size_prof(i)=!indef
    ENDELSE
  ENDELSE
;  print,count_flux,count_back,count_prof
ENDFOR

;pass flux into Jy
omega_pix=ABS(sxpar(h,'CDELT1'))*ABS(sxpar(h,'CDELT2'))/!radeg/!radeg
all_Flux=all_Flux*omega_pix*1.e6
all_stflux=all_stflux*omega_pix*1.e6
all_back=all_back*omega_pix*1.e6
all_stback=all_stback*omega_pix*1.e6
all_size=all_size*ABS(sxpar(h,'CDELT1'))*60. ;arcmin
all_size_prof=all_size_prof*ABS(sxpar(h,'CDELT1'))*60. ;arcmin

IF NOT keyword_set(no_plot) THEN BEGIN
    !p.multi=[0,2,2]
    ind=where(all_Nback NE 0,count)
    IF keyword_set(xrange) THEN xr=xrange ELSE xr=minmax(all_size(ind))
    IF keyword_set(yrange) THEN BEGIN
        yr=yrange
    ENDIF ELSE BEGIN
        yr=minmax([all_Flux(ind),all_back(ind)*all_Nflux(ind), $
                   all_Flux(ind)+all_back(ind)*all_Nflux(ind)])
    ENDELSE           
    plot,all_size(ind),all_Flux(ind),yr=yr,/ysty, $
      xtit='radius in arcmin',ytit='Flux (Jy)',psym=-4,title=title, $
      xr=xr,/xsty
;    err_back=2.*all_stback(ind)/sqrt(all_Nback(ind))*all_Nflux(ind)
    err_bar,all_size(ind),all_Flux(ind),yrms=2.*all_stflux(ind)
    oplot,all_size(ind),all_back(ind)*all_Nflux(ind),psym=-5,color=100
    oplot,all_size(ind),all_Flux(ind)+all_back(ind)*all_Nflux(ind),psym=-3,color=150

    plot,all_size(ind),all_Nflux(ind),psym=-4, $
      xtit='radius in arcmin',ytit='Npix',xr=xr,/xsty
    oplot,all_size(ind),all_Nback(ind),color=100,psym=-5

    IF keyword_set(log_plot) THEN BEGIN
      plot_oo,all_size_prof(ind),avg_prof(ind),psym=-4, $
        xtit='radius in arcmin',ytit='Avg profile (MJy/sr)', $
        xr=xr,/xsty
    ENDIF ELSE BEGIN
      plot,all_size_prof(ind),avg_prof(ind),psym=-4, $
        xtit='radius in arcmin',ytit='Avg profile (MJy/sr)', $
        xr=xr,/xsty
    ENDELSE
ENDIF

flux_radius=all_size
prof_radius=all_size_prof

!p.multi=old_pmulti

sortie:

END
