PRO mosaique3,al,del,d,h,w,res,weight,undef=undef, $
              silent=silent,deg_res=deg_res,round=round,lbl=lbl
;+
; NAME:
;       mosaique3
; CALLING SEQUENCE:
;       mosaique3,al,del,d,h,w,res,weight
; PURPOSE:
;       create a composite image of image res (big image) and image in
;	d (small image).
;	Coordinates are derived from their headers and projected coreclty
;	res might already have defined values, in which case (res+d)/2.
;	is set in the overlaping regions
;	interpolation is done bilinearily (no averaging in a beam)
; (you cal also try PMlib's mosaic.pro which uses gaussian convolution)
; INPUTS:
;      al,del = alpha & delta coord of each pix of big image (res). 2 D arrays
;      d,h    = data and header of the small image
;      w      = weight (scalar or 2D) of the input image d
;      weight = current weight (2D) for res image
;               (used as a weight in summing)
; OPTIONAL INPUT:
;      undef  = undefined value
;      silent = if set routine is silent
;      deg_res= if set the small image's angular resolution is first degraded
;	        to deg_res (deg)
; OUTPUTS:
;      res    = res+small image, weight is updated
; PROCEDURE
;	1/ input coordinates are transformed into pixels of the small image
;	   input coordinates are considered to be of the type given in h
;	   (accepted are RA, GLON, ECLON)
;	2/ If requested, the angular resolution of the small image is
;	   first degraded using degrade_res.pro
;	3/ The small image is projected onto the big one using my_bilinear2
;	4/ The so-projected image is added to res
;	   res might already have defined values, in which case (res+d)/2.
;	   is set in the overlaping regions
; SUBROUTINE USED
;       my_bilinear2,sxpar,degrade_res
; RESTRICTIONS
;       when adding the 1st map, res should be all set to !indef
;       and weight to 0.
; MODIFICATION HISTORY:
;       written JPB 15-08-92
;        modified JPB Aug 23 1996 to include individual weight w
;        modified PM  Mar 08 2001 to enhance indef handling and make 2D weight
;-

@proj_common.com

IF N_PARAMS(0) LT 6 THEN begin
  print,'Calling Sequence: mosaique3,alpha,delta,data,header,w,res,weight'
  print,'Accepted k-words: ,undef=,/silent,deg_res=deg_res,/round,/lbl'
  print,'(for mosaicing using gaussian convol., use PMLIB mosaic.pro)'
  goto,sortie
ENDIF

IF not keyword_set(undef) THEN BEGIN
  undef=!indef
ENDIF

;convert given alpha delta into pixels in the small image
IF not keyword_set(silent) THEN BEGIN
  print,'I am now converting your input coordinates into pixels'+ $
   ' of the small image'
ENDIF
h2pix,h,/silent
;!!! A decouper d'apres 4 coins de la petite image
coo2pix2,al,del,xpix,ypix,phi,theta,/silent,lbl=lbl

;If necessary, degrade resolution of the small image first
IF keyword_set(deg_res) THEN BEGIN
  IF not keyword_set(silent) THEN BEGIN
    print,'I am now degrading the resolution of the small image'
  ENDIF
  d=degrade_res(d,h,deg_res,/remove_border,indef=undef)
ENDIF

;indpos=where(theta LT 0.,countpos)
;xpix=xpix(indpos) & ypix=ypix(indpos)
;=== This is to set xpix,ypix outside the small image (-1) for points
;=== on the wrong side of the sphere (for e.g. TAN projection)
;indpos=where(theta LT 0.,countpos)
mapsiz=max(abs([1.*sxpar(h,'NAXIS1')*sxpar(h,'CDELT1'),1.*sxpar(h,'NAXIS2')*sxpar(h,'CDELT2')]))/!radeg*sqrt(2.)
IF defined(theta) THEN BEGIN
  indpos=where(!pi-2.*theta GT mapsiz,countpos)
  ;stop
  IF countpos NE 0 THEN BEGIN
    xpix(indpos)=-1
    ypix(indpos)=-1
  ENDIF
ENDIF

;stop
;tvscl,sqrt(xpix^2+ypix^2)>0<500

;compute a big image of the small image with reprojection
IF not keyword_set(silent) THEN BEGIN
  print,'I am now interpolating for these pixels into the small image'
ENDIF
IF keyword_set(round) THEN BEGIN
  xpix=round(xpix) & ypix=round(ypix)
ENDIF
;res1=my_bilinear(d,xpix,ypix,silent=silent)
res1=my_bilinear2(d,xpix,ypix,silent=silent)

IF not keyword_set(silent) THEN BEGIN
  print,'I am now adding the interpolated values to the result'
ENDIF

;make a weight map for small image...
CASE n_elements(w) OF
  0 : BEGIN
    ww = res
    ww(*) = 1.
  END
  1 : BEGIN
    ww = res
    ww(*) = w
  END
  n_elements(res) : BEGIN
    ww = res
    ww(*) = w
  END
  ELSE : message,'Weight and Map sizes are inconsistent!'
ENDCASE

;find undefined values in res1 (!indef) and res (undef)
;and turn their weight into 0 to ignore them during the sum
ind = where(NOT float(finite([res])) OR res EQ !indef OR res EQ undef,count)
IF count GT 0 THEN BEGIN
  res(ind) = 0.
  weight(ind) = 0.
ENDIF
ind = where(NOT float(finite([weight])) OR weight EQ !indef OR weight EQ undef,count)
IF count GT 0 THEN BEGIN
  res(ind) = 0.
  weight(ind) = 0.
ENDIF
ind = where(NOT float(finite([res1])) OR res1 EQ !indef OR res1 EQ undef,count)
IF count GT 0 THEN BEGIN
  res1(ind) = 0.
  ww(ind) = 0.
ENDIF
ind = where(NOT float(finite([ww])) OR ww EQ !indef OR ww EQ undef,count)
IF count GT 0 THEN BEGIN
  res1(ind) = 0.
  ww(ind) = 0.
ENDIF

;sum the maps
;ind=where(res NE undef and res1 NE undef,count)
;IF count NE 0 THEN BEGIN
;  res(ind)=(weight(ind)*res(ind)+w*res1(ind))/(weight(ind)+w)
;  weight(ind)=weight(ind)+w
res = (weight*res+ww*res1)/(weight+ww)
weight = weight+ww
;ENDIF

;reset indef values (null weight) to !indef...
;ind=where(res EQ undef and res1 NE undef,count)
ind = where(weight EQ 0, count)
IF count GT 0 THEN BEGIN
;  res(ind)=res1(ind)
  res(ind) = !indef
;;  weight(ind)=w
;  weight(ind)=ww(ind)
ENDIF

;== reset undef values in res to !indef
ind=where(res EQ undef,count)
IF count NE 0 THEN BEGIN
  res(ind)=!indef
ENDIF
sortie:
return
END
