function sum_box,im,head,X0,Y0,db,back,stflux,stback,Nflux,Nback, $
                 dx,dy,silent=silent,dist=dist,d_value=d_value

;+
; NAME:
;       sum_box
; CALLING SEQUENCE:
;       sum=sum_box(im,head,X0,Y0,db,back,stflux,stback,Nflux,Nback,dx,[dy])
; PURPOSE:
;	Compute the integral into a rectangular box around pixel X0,Y0
;	and subtract the background computed into a surrounding region.
; INPUTS:
;	im,head	= image and header.
;	X0,Y0	= center pixel coordinates.
;	dx	= 1/2 x size of the window.
;	dy	= 1/2 y size of the window
;	db	= thickness of the background region.
; OPTIONAL INPUT:
;	dy	= y size of the window
; OUTPUTS:
;       Result of the function	= Flux in the box (background subtracted)
;	back	= computed background
;       stflux	= Flux standard deviation (in flux box)
;       stback	= back standard deviation (in back box)
;	Nflux   = Number of defined pixels in Flux region
;	Nback   = Number of defined pixels in back region
; OPTIONAL PARAMETERS
;	dist    = If set, boxes are choosed along this distance
; SUBROUTINE AND PROCEDURE USED
;	The background is computing by averaging in a region of db pixels
;	around the flux region.
;	The flux is computed as the sum of the image in the flux region
;	(2*dx+1,2*dy+1 wide box around x0,y0) and subtracting (2*dx+1)*
;	(2*dy+1)*back.
;
;	moment
; SIDE EFFECTS:
; MODIFICATION HISTORY:
;       WRITTEN, J.P. Bernard, 7-Sep-1992
;       Modified JPB Dec 3 1996 to include dist
;-

if n_params() LT 11 then begin
  print,'truc=sum_box(image,header,X0,Y0,db,background,stflux,stback,'
  print,'             Nflux,Nback,dx,[dy])'
  print,'Accepted Key-Words: /silent,dist='
  RETURN,1
  GOTO,sortie
endif

si=size(im) & Nx=si(1) & Ny=si(2)

IF not keyword_set(dist) THEN BEGIN
  dist=intarr(Nx,Ny)
  FOR i=0,Nx-1 DO BEGIN
    FOR j=0,Ny-1 DO BEGIN
      dist(i,j)=max(abs([i-x0,j-y0]))
    ENDFOR
  ENDFOR
ENDIF

;if n_params() EQ 11 AND not keyword_set(dist) THEN BEGIN
;  dy=dx
;  IF n_elements(silent) EQ 0 THEN BEGIN
;    print,'Square region assumed'
;  ENDIF
;ENDIF
;
;Compute the background
;IF not keyword_set(dist) THEN BEGIN
;  xbeg=x0-dx & xend=x0+dx
;  ybeg=y0-dy & yend=y0+dy
;  xdeb=xbeg & ydeb=ybeg
;  xfin=xend & yfin=yend
;  IF xbeg LT 0 THEN xdeb=0
;  IF ybeg LT 0 THEN ydeb=0
;  IF xend GT Nx-1 THEN xfin=Nx-1
;  IF yend GT Ny-1 THEN yfin=Ny-1
;  imcopy=im
;  imcopy(xdeb:xfin,ydeb:yfin)=!indef
;  xbeg=x0-dx-db & xend=x0+dx+db
;  ybeg=y0-dy-db & yend=y0+dy+db
;  xdeb=xbeg & ydeb=ybeg
;  xfin=xend & yfin=yend
;  IF xbeg LT 0 THEN xdeb=0
;  IF ybeg LT 0 THEN ydeb=0
;  IF xend GT Nx-1 THEN xfin=Nx-1
;  IF yend GT Ny-1 THEN yfin=Ny-1
;  back_zone=imcopy(xdeb:xfin,ydeb:yfin)
;ENDIF ELSE BEGIN
  back_zone=[!indef]
  ind=where(dist GT dx AND dist LE dx+db,count)
  IF count NE 0 THEN BEGIN
    back_zone=im(ind)
  ENDIF
;ENDELSE

ind=where(back_zone NE !indef,count) & Nback=count
IF count EQ 0 THEN BEGIN
  IF not keyword_set(silent) THEN print,'All pixels of the background zone are undefined'
  goto,sortie
ENDIF ELSE BEGIN
  if not keyword_set(silent) THEN BEGIN
    print,'Background region has ',count,' defined pixels'
  ENDIF
ENDELSE
IF count GT 1 THEN BEGIN
  IF min(back_zone(ind)) NE max(back_zone(ind)) THEN BEGIN
    stdd=moment(back_zone(ind))
    std=sqrt(stdd(1)) & back=stdd(0)
;    std=stdev(back_zone(ind),back)
    stback=std
  ENDIF ELSE BEGIN
    stback=!indef
    back=avg(back_zone(ind))
  ENDELSE
ENDIF ELSE BEGIN
  stback=!indef & back=back_zone(ind)
ENDELSE
if not keyword_set(silent) THEN BEGIN
  print,'Background= ',back,' +- ',stback
ENDIF
countback=count

;Compute the flux
;IF not keyword_set(dist) THEN BEGIN
;  xbeg=x0-dx & ybeg=y0-dy
;  xend=x0+dx & yend=y0+dy
;  xdeb=xbeg & xfin=xend
;  ydeb=ybeg & yfin=yend
;  IF xbeg LT 0 THEN xdeb=0
;  IF ybeg LT 0 THEN ydeb=0
;  IF xend GT Nx-1 THEN xfin=Nx-1
;  IF yend GT Ny-1 THEN yfin=Ny-1
;  flux_zone=im(xdeb:xfin,ydeb:yfin)
;  nfz=(xfin-xdeb+1)*(yfin-ydeb+1)
;ENDIF ELSE BEGIN
  flux_zone=[!indef] & dist_zone=[!indef]
  ind=where(dist LE dx,count)
  IF count NE 0 THEN BEGIN
    flux_zone=im(ind)
    dist_zone=dist(ind)
  ENDIF
  nfz=count
;ENDELSE

ind=where(flux_zone NE !indef,count) & Nflux=count
npix=count
IF (count EQ 0) THEN BEGIN
  if not keyword_set(silent) THEN print,'integration region has only undefined values'
  goto,sortie
ENDIF ELSE BEGIN
  if not keyword_set(silent) THEN BEGIN
    print,'Integration region has',npix,' pixels.'
    print,'Integration region has',nfz-npix,' undefined pixels.'
  ENDIF
ENDELSE
;when Moment can be used
IF npix GT 1 THEN BEGIN
  stdd=moment(flux_zone(ind))
  mmm=stdd(0) & std=sqrt(stdd(1))
;  std=stdev(flux_zone(ind),mmm)
  flux=mmm & stflux=std
  flux=flux*npix
ENDIF ELSE BEGIN
  stflux=!indef
  flux=total(flux_zone(ind))
ENDELSE
d_value=avg(dist_zone(ind))
;print,d_value

;Subtract background
;sum=flux-(2*dx+1)*(2*dy+1)*back
sum=flux-npix*back
if n_elements(silent) EQ 0 THEN BEGIN
  print,'unsubtracted Flux= ',flux,' +- ',stflux
  print,'  subtracted Flux= ',sum,' +- ',stflux
ENDIF

return,sum

sortie:

end

