FUNCTION zmy,a,head,siz,headout, $
             centcoord=centcoord,centpix=centpix,new_proj=new_proj,new_coord=new_coord, $
             silent=silent,previous_lut=previous_lut

;+
; NAME:
;       zmy
; CALLING SEQUENCE:
;       zmarray=zmy(image,head,siz[,hout]
;		   [,centcoord=[]][,centpix=[]][,/previous_lut)
; PURPOSE:
;       zoom an image in various coordinate system/projection.
;	A new header is generated for the zoomed array
; INPUTS:
;	image, head     = original image and header
;     siz            = size of the area (degrees). Can be a 2 elt set
; OPTIONAL INPUT:
;	centcoord	= a set giving center coord
;			  in degrees in the NEW coordinate system
;	centpix		= a set giving center pixel
;			  in the original image
;	new_proj        = to set a new projection type
;	new_coord       = to set a new coordinate system
;	previous_lut    = use the previous Look-up table projection
; OUTPUTS:
;	hout		= header of the zoomed image
; PROCEDURE AND SUBROUTINE USED
;	A zoomed image header is produced, according to the coordinates,
;	coordinate system and projection specified.
;	The result image is obtained as the projection of the input
;	corresponding to this new header.
;
;       pix2coo2,coo2pix2,h2pix,project2
; COMMONS:
;     @proj_common.com, @lut_proj.com 
; SIDE EFFECTS:
;       ?
; MODIFICATION HISTORY:
;       written JPB 05-08-92
;	Modified JPB 22-01-93 to use convert_ad_pix instead of cor2pix
;				and create a better output header
;	Modified 19-Mar-93 to put header center @ image center
;	Modified Dec 12 1993 to allow deal with l,b maps and cube inputs
;	Modified MAR 22 1995 to Generalize coordinate conversion
;-

@proj_common.com
@lut_proj.com

;on_error,2
indef=-32768.
d2r=!pi/180.

IF N_PARAMS(0) EQ 0 THEN begin
  print,'res=zmy(image,head,siz(deg)[,headout])'
  print,'Accepted Key-Words: ,centcoord=[,],centpix=[,][,/silent]'
  goto,sortie
ENDIF

;============== Check input parameters
IF N_PARAMS(0) LT 3 THEN begin
  PRINT,'missing parameters'
  goto,sortie
ENDIF
IF N_PARAMS(0) EQ 3 THEN begin
  print,'No modified header will be created'
ENDIF
IF n_elements(centcoord) EQ 0 and n_elements(centpix) EQ 0 THEN BEGIN
  print,'centcoord and centpix K-words are exclusive'
  goto,sortie
ENDIF
IF n_elements(centcoord) NE 0 and n_elements(centpix) NE 0 THEN BEGIN
  print,'One of centcoord and centpix K-words has to be set'
  goto,sortie
ENDIF
IF n_elements(centcoord) NE 0 THEN BEGIN
  si=size(centcoord)
  IF si(0) NE 1 THEN BEGIN
    print,'centcoord should have 1 or 2 elements'
    goto,sortie
  ENDIF
ENDIF
IF n_elements(centpix) NE 0 THEN BEGIN
  si=size(centpix)
  IF si(0) NE 1 THEN BEGIN
    print,'centpix should have 1 or 2 elements'
    goto,sortie
  ENDIF
ENDIF
si=size(siz)
IF si(0) GT 1 THEN BEGIN
  print,'siz should be 1 or 2 elements only'
  goto,sortie
ENDIF
IF si(0) EQ 0 THEN BEGIN
  print,'square area assumed'
  six=siz
  siy=siz
ENDIF
IF keyword_set(previous_lut) and $
  (keyword_set(new_proj) OR keyword_set(new_coord)) THEN BEGIN
  print,'zmy:/previous_lut and new_coord/new_proj are exclusive'
  goto,sortie
ENDIF

IF si(0) EQ 1 THEN BEGIN
  six=siz(0)
  siy=siz(1)
ENDIF
;determine the array size
si=size(a)
IF si(0) NE 2 and si(0) NE 3 THEN BEGIN
  print,'image should be a 2D or 3D'
  goto,sortie
ENDIF
IF keyword_set(previous_lut) THEN BEGIN
  x=last_x_proj
  y=last_y_proj
  headout=last_headout
  goto,interp
ENDIF

IF si(0) EQ 2 THEN nimage=1
IF si(0) EQ 3 THEN nimage=si(3)

;============= Reads the projection values from the header
h2pix,head,silent=silent
IF proj EQ 'CSC' THEN BEGIN
  print,'zmy:Warning trying to zoom into a Cube.'
ENDIF
ct=sxpar(head,'CTYPE1')
cor_proj_dec,ct,old_coord,old_proj,/silent
IF not keyword_set(new_proj) THEN BEGIN
  new_proj=old_proj
ENDIF
IF not keyword_set(new_coord) THEN BEGIN
  new_coord=old_coord
ENDIF

;============= determine the pixel corresponding to the field center
IF keyword_set(centcoord) THEN BEGIN
  IF new_coord NE old_coord THEN BEGIN	;change center coo to old header
    inco=coo2co(new_coord)
    outco=coo2co(old_coord)
    vec=fltarr(1,2)
    vec(0,0)=centcoord(0) & vec(0,1)=centcoord(1)
    coox=centcoord(0) & cooy=centcoord(1)
    vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
    c0=vec(0,0) & c1=vec(0,1)
    coo2pix2,c0,c1,x1,y1,silent=silent
;    IF not keyword_set(silent) THEN BEGIN
;      print,'x1=',x1,' y1=',y1
;      print,'coox=',c0,' cooy=',c1,'  coord=',old_coord
;      print,'coox=',coox,' cooy=',cooy,'  coord=',new_coord
;    ENDIF
  ENDIF ELSE BEGIN
    c0=centcoord(0) & c1=centcoord(1)
    coox=centcoord(0) & cooy=centcoord(1)
    coo2pix2,c0,c1,x1,y1,silent=silent
;    IF not keyword_set(silent) THEN BEGIN    
;      print,'x1=',x1,' y1=',y1
;      print,'coox=',c0,' cooy=',c1,'  coord=',old_coord
;    ENDIF  
  ENDELSE
ENDIF ELSE BEGIN
  x1=centpix(0)
  y1=centpix(1)
  pix2coo2,x1,y1,c1,c2,silent=silent
  IF new_coord NE old_coord THEN BEGIN	;change center coo to old header
    inco=coo2co(old_coord)
    outco=coo2co(new_coord)
    vec=fltarr(1,2)
    vec(0,0)=c1 & vec(0,1)=c2
    vec=my_coorconv(vec,infmt='l',outfmt='l',inco=inco,outco=outco)
    coox=vec(0,0) & cooy=vec(0,1)
;   IF not keyword_set(silent) THEN BEGIN    
;      print,'x1=',x1,' y1=',y1
;      print,'coox=',c1,' cooy=',c2,'  coord=',old_coord
;      print,'coox=',coox,' cooy=',cooy,'  coord=',new_coord
;   ENDIF
  ENDIF ELSE BEGIN
    coox=c1 & cooy=c2
;   IF not keyword_set(silent) THEN BEGIN        
;      print,'x1=',x1,' y1=',y1
;      print,'coox=',c1,' cooy=',c2,'  coord=',old_coord
;   ENDIF    
  ENDELSE
ENDELSE
IF keyword_set(new_proj) THEN BEGIN
  old_proj=new_proj
  IF not keyword_set(silent) THEN print,'switching to ',new_proj
ENDIF
IF keyword_set(new_coord) THEN BEGIN
  old_coord=new_coord
  IF not keyword_set(silent) THEN print,'switching to ',new_coord
ENDIF

;============= construct the header for the zoomed image
headout=head
new_tx=round(ABS(six/cdelt1*d2r))
new_ty=round(ABS(siy/cdelt2*d2r))
new_centx=(1.*new_tx+1.)/2.
new_centy=(1.*new_ty+1.)/2.
new_cor1=coox
new_cor2=cooy
sxaddpar,headout,'CRPIX1',new_centx
sxaddpar,headout,'CRPIX2',new_centy
sxaddpar,headout,'CRVAL1',new_cor1
sxaddpar,headout,'CRVAL2',new_cor2
sxaddpar,headout,'NAXIS1',new_tx
sxaddpar,headout,'NAXIS2',new_ty
cor_proj_cod,old_coord,old_proj,ct1,ct2
sxaddpar,headout,'CTYPE1',ct1
sxaddpar,headout,'CTYPE2',ct2
headout=fix_radec_header(headout)

;============= reproject image into new header
interp:
res=project2(head,a,headout,silent=silent,previous_lut=previous_lut)

return,res
sortie:

END
